CREATE TABLE sscOrder (
  orderID INT(11) NOT NULL AUTO_INCREMENT,
  customerID INT(11) DEFAULT NULL,
  userName VARCHAR(100) DEFAULT NULL,
  orderNumber INT(11) DEFAULT NULL,
  trackingNumber VARCHAR(100) DEFAULT NULL,
  total DECIMAL(10,4) DEFAULT NULL,
  shipping DECIMAL(10,4) DEFAULT NULL,
  tax DECIMAL(10,4) DEFAULT NULL,
  taxableSubtotal DECIMAL(10,4) DEFAULT NULL,
  subtotal DECIMAL(10,4) DEFAULT NULL,
  weight DECIMAL(10,4) DEFAULT NULL,
  quantity INT(11) DEFAULT NULL,
  created VARCHAR(100) DEFAULT NULL,
  lastModified VARCHAR(100) DEFAULT NULL,
  completed VARCHAR(100) DEFAULT NULL,
  autoSavedCartToken VARCHAR(250) DEFAULT NULL,
  status VARCHAR(100) DEFAULT NULL,
  statusDetails VARCHAR(250) DEFAULT NULL,
  shippingMethod VARCHAR(250) DEFAULT NULL,
  firstName VARCHAR(100) DEFAULT NULL,
  lastName VARCHAR(100) DEFAULT NULL,
  organization VARCHAR(100) DEFAULT NULL,
  address1 VARCHAR(250) DEFAULT NULL,
  address2 VARCHAR(250) DEFAULT NULL,
  city VARCHAR(100) DEFAULT NULL,
  state VARCHAR(100) DEFAULT NULL,
  postalCode VARCHAR(100) DEFAULT NULL,
  country VARCHAR(100) DEFAULT NULL,
  phone1 VARCHAR(100) DEFAULT NULL,
  phone2 VARCHAR(100) DEFAULT NULL,
  email1 VARCHAR(100) DEFAULT NULL,
  email2 VARCHAR(100) DEFAULT NULL,
  paymentProfileID varchar(100) DEFAULT NULL,
  extra1 VARCHAR(255) DEFAULT NULL,
  extra2 VARCHAR(255) DEFAULT NULL,
  extra3 VARCHAR(255) DEFAULT NULL,
  extra4 VARCHAR(255) DEFAULT NULL,
  extra5 VARCHAR(255) DEFAULT NULL,
  notes text DEFAULT NULL,
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.order.OrderBean',
  PRIMARY KEY  (orderID)
);

CREATE INDEX IDX_sscOrder_userName ON sscOrder (userName);
CREATE INDEX IDX_sscOrder_orderNumber ON sscOrder (orderNumber);
CREATE INDEX IDX_sscOrder_trackingNumber ON sscOrder (trackingNumber);
CREATE INDEX IDX_sscOrder_firstName ON sscOrder (firstName);
CREATE INDEX IDX_sscOrder_lastName ON sscOrder (lastName);
CREATE INDEX IDX_sscOrder_address1 ON sscOrder (address1);
CREATE INDEX IDX_sscOrder_postalCode ON sscOrder (postalCode);
CREATE INDEX IDX_sscOrder_phone1 ON sscOrder (phone1);
CREATE INDEX IDX_sscOrder_email1 ON sscOrder (email1);
CREATE INDEX IDX_sscO_autoSavedCartToken ON sscOrder (autoSavedCartToken);
CREATE INDEX IDX_sscOrder_completed ON sscOrder (completed);

CREATE TABLE sscOrderDelivery (
  orderDeliveryID INT(11) NOT NULL AUTO_INCREMENT,
  orderID INT(11) NOT NULL DEFAULT '0',
  trackingNumber VARCHAR(100) DEFAULT NULL,
  total DECIMAL(10,4) DEFAULT NULL,
  shipping DECIMAL(10,4) DEFAULT NULL,
  tax DECIMAL(10,4) DEFAULT NULL,
  taxableSubtotal DECIMAL(10,4) DEFAULT NULL,
  subtotal DECIMAL(10,4) DEFAULT NULL,
  weight DECIMAL(10,4) DEFAULT NULL,
  quantity INT(11) DEFAULT NULL,
  lastModified VARCHAR(100) DEFAULT NULL,
  delivered VARCHAR(100) DEFAULT NULL,
  status VARCHAR(100) DEFAULT NULL,
  statusDetails VARCHAR(250) DEFAULT NULL,
  firstName VARCHAR(100) DEFAULT NULL,
  lastName VARCHAR(100) DEFAULT NULL,
  organization VARCHAR(100) DEFAULT NULL,
  address1 VARCHAR(250) DEFAULT NULL,
  address2 VARCHAR(250) DEFAULT NULL,
  city VARCHAR(100) DEFAULT NULL,
  state VARCHAR(100) DEFAULT NULL,
  postalCode VARCHAR(100) DEFAULT NULL,
  country VARCHAR(100) DEFAULT NULL,
  phone1 VARCHAR(100) DEFAULT NULL,
  phone2 VARCHAR(100) DEFAULT NULL,
  email1 VARCHAR(100) DEFAULT NULL,
  email2 VARCHAR(100) DEFAULT NULL,
  extra1 VARCHAR(255) DEFAULT NULL,
  extra2 VARCHAR(255) DEFAULT NULL,
  extra3 VARCHAR(255) DEFAULT NULL,
  extra4 VARCHAR(255) DEFAULT NULL,
  extra5 VARCHAR(255) DEFAULT NULL,
  notes text DEFAULT NULL,
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.order.OrderDeliveryBean',
  PRIMARY KEY  (orderDeliveryID),
  KEY orderIDIndex (orderID),
  FOREIGN KEY (`orderID`) REFERENCES `sscOrder` (`orderID`) ON DELETE CASCADE
);

CREATE INDEX IDX_sscOrderDelivery_trackingNumber ON sscOrderDelivery (trackingNumber);
CREATE INDEX IDX_sscOrderDelivery_firstName ON sscOrderDelivery (firstName);
CREATE INDEX IDX_sscOrderDelivery_lastName ON sscOrderDelivery (lastName);
CREATE INDEX IDX_sscOrderDelivery_address1 ON sscOrderDelivery (address1);
CREATE INDEX IDX_sscOrderDelivery_postalCode ON sscOrderDelivery (postalCode);
CREATE INDEX IDX_sscOrderDelivery_phone1 ON sscOrderDelivery (phone1);
CREATE INDEX IDX_sscOrderDelivery_email1 ON sscOrderDelivery (email1);

CREATE TABLE sscOrderItem (
  orderItemID INT(11) NOT NULL AUTO_INCREMENT,
  orderID INT(11) NOT NULL DEFAULT '0',
  orderDeliveryID INT(11) NOT NULL DEFAULT '0',
  total DECIMAL(10,4) DEFAULT NULL,
  weight DECIMAL(10,4) DEFAULT NULL,
  costTotal DECIMAL(10,4) DEFAULT NULL,
  quantity INT(11) NOT NULL DEFAULT '1',
  productID INT(11) NOT NULL DEFAULT '0',
  manufacturerID INT(11) DEFAULT NULL,
  code VARCHAR(100) NOT NULL DEFAULT '',
  seoCode VARCHAR(255) DEFAULT NULL,
  name VARCHAR(255) DEFAULT NULL,
  isActive tinyint(4) NOT NULL DEFAULT '1',
  unitCost DECIMAL(10,4) DEFAULT NULL,
  unitPrice DECIMAL(10,4) DEFAULT NULL,
  altPrice DECIMAL(10,4) DEFAULT NULL,
  productWeight DECIMAL(10,4) DEFAULT NULL,
  isTaxed tinyint(4) NOT NULL DEFAULT '0',
  image VARCHAR(255) DEFAULT NULL,
  shortDescription VARCHAR(255) DEFAULT NULL,
  extra1 VARCHAR(255) DEFAULT NULL,
  extra2 VARCHAR(255) DEFAULT NULL,
  extra3 VARCHAR(255) DEFAULT NULL,
  extra4 VARCHAR(255) DEFAULT NULL,
  extra5 VARCHAR(255) DEFAULT NULL,
  decrementedSKUs TEXT,
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.order.OrderItemBean',
  PRIMARY KEY  (orderItemID),
  KEY orderIDIndex (orderID),
  KEY orderDeliveryIDIndex (orderDeliveryID),
  FOREIGN KEY (`orderID`) REFERENCES `sscOrder` (`orderID`) ON DELETE CASCADE,
  FOREIGN KEY (`orderDeliveryID`) REFERENCES `sscOrderDelivery` (`orderDeliveryID`) ON DELETE CASCADE
);

CREATE INDEX IDX_sscOrderItem_code ON sscOrderItem (code);
CREATE INDEX IDX_sscOrderItem_name ON sscOrderItem (name);
ALTER TABLE sscDiscount ADD FOREIGN KEY (orderItemID) REFERENCES sscOrderItem (orderItemID) ON DELETE SET NULL;

CREATE TABLE sscOrderItemAttribute (
  orderItemAttributeID INT(11) NOT NULL AUTO_INCREMENT,
  orderItemID INT(11) NOT NULL DEFAULT '0',
  total DECIMAL(10,4) DEFAULT NULL,
  weight DECIMAL(10,4) DEFAULT NULL,
  costTotal DECIMAL(10,4) DEFAULT NULL,
  attributeValue TEXT NOT NULL,
  attributeID INT(11) NOT NULL DEFAULT '0',
  attributeCode VARCHAR(100) NOT NULL DEFAULT '',
  attributeName VARCHAR(250) DEFAULT NULL,
  attributeUnitCost DECIMAL(10,4) DEFAULT NULL,
  attributeUnitPrice DECIMAL(10,4) DEFAULT NULL,
  attributeAltPrice DECIMAL(10,4) DEFAULT NULL,
  attributeWeight DECIMAL(10,4) DEFAULT NULL,
  attributeImage VARCHAR(255) DEFAULT NULL,
  optionID INT(11) DEFAULT NULL,
  optionCode VARCHAR(100) DEFAULT NULL,
  optionName VARCHAR(250) DEFAULT NULL,
  optionUnitCost DECIMAL(10,4) DEFAULT NULL,
  optionUnitPrice DECIMAL(10,4) DEFAULT NULL,
  optionAltPrice DECIMAL(10,4) DEFAULT NULL,
  optionWeight DECIMAL(10,4) DEFAULT NULL,
  optionImage VARCHAR(255) DEFAULT NULL,
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.order.OrderItemAttributeBean',
  PRIMARY KEY  (orderItemAttributeID),
  KEY orderItemIDIndex (orderItemID),
  FOREIGN KEY (`orderItemID`) REFERENCES `sscOrderItem` (`orderItemID`) ON DELETE CASCADE
);

CREATE INDEX IDX_sscOrderItemAttribute_code ON sscOrderItemAttribute (attributeCode);
CREATE INDEX IDX_sscOrderItemAttribute_name ON sscOrderItemAttribute (attributeName);
CREATE INDEX IDX_sscOrderItemAttribute_optionCode ON sscOrderItemAttribute (optionCode);
CREATE INDEX IDX_sscOrderItemAttribute_optionName ON sscOrderItemAttribute (optionName);

CREATE TABLE sscCountry (
  countryID INT(11) NOT NULL AUTO_INCREMENT,
  name VARCHAR(255) NOT NULL DEFAULT '0',
  code VARCHAR(8) DEFAULT NULL,
  countryOrder INT(11) NOT NULL DEFAULT '0',
  allowOtherStates TINYINT(4) NOT NULL DEFAULT '1',
  isActive TINYINT(4) NOT NULL DEFAULT '1',
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.order.CountryBean',
  PRIMARY KEY  (countryID),
  UNIQUE KEY countryCodeUKey (code)
);

CREATE INDEX IDX_sscCountry_name ON sscCountry (name);
CREATE INDEX IDX_sscCountry_order ON sscCountry (countryOrder);

CREATE TABLE sscState (
  stateID INT(11) NOT NULL AUTO_INCREMENT,
  countryID INT(11) NOT NULL DEFAULT '0',
  name VARCHAR(255) NOT NULL DEFAULT '',
  code VARCHAR(16) DEFAULT NULL,
  stateOrder INT(11) NOT NULL DEFAULT '0',
  isActive TINYINT(4) NOT NULL DEFAULT '1',
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.order.StateBean',
  PRIMARY KEY  (stateID),
  UNIQUE KEY stateCodeUKey (code, countryID),
  KEY countryIDIndex (countryID),
  FOREIGN KEY (`countryID`) REFERENCES `sscCountry` (`countryID`) ON DELETE CASCADE
);

CREATE INDEX IDX_sscState_name ON sscState (name);
CREATE INDEX IDX_sscState_order ON sscState (stateOrder);


CREATE TABLE sscOrderDiscount (  
  orderDiscountID INT(11) NOT NULL AUTO_INCREMENT,
  orderID INT(11) NOT NULL DEFAULT '0',
  orderItemID INT(11) NULL,
  total DECIMAL(10,4) DEFAULT NULL,
  discountID INT(11) NOT NULL DEFAULT '0',
  code VARCHAR(100) NOT NULL DEFAULT '',
  name VARCHAR(255) DEFAULT NULL,
  usersAppliesType VARCHAR(32) DEFAULT NULL,
  usersAppliesValue VARCHAR(255) DEFAULT NULL,
  itemsAppliesType VARCHAR(32) DEFAULT NULL,
  itemsAppliesValue VARCHAR(255) DEFAULT NULL,
  cartItemsType VARCHAR(32) DEFAULT NULL,
  cartItemsValue VARCHAR(255) DEFAULT NULL,
  rewardType VARCHAR(32) DEFAULT NULL,
  reward DECIMAL(10,4) NOT NULL DEFAULT '0',
  couponCode VARCHAR(100) DEFAULT NULL,
  floor DECIMAL(10,4) DEFAULT NULL,
  ceiling DECIMAL(10,4) DEFAULT NULL,
  extra1 VARCHAR(255) DEFAULT NULL,
  extra2 VARCHAR(255) DEFAULT NULL,
  extra3 VARCHAR(255) DEFAULT NULL,
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.order.OrderDiscountBean',
  PRIMARY KEY  (orderDiscountID),
  KEY orderIDIndex (orderID),
  FOREIGN KEY (`orderID`) REFERENCES `sscOrder` (`orderID`) ON DELETE CASCADE
);

CREATE INDEX IDX_sscOrderDiscount_total ON sscOrderDiscount (total);
CREATE INDEX IDX_sscOrderDiscount_code ON sscOrderDiscount (code);
CREATE INDEX IDX_sscOrderDiscount_name ON sscOrderDiscount (name);

CREATE TABLE sscOrderNumber (
  orderNumber INT(11) NOT NULL AUTO_INCREMENT,
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.order.OrderNumber',
  PRIMARY KEY (orderNumber)
);

CREATE TABLE sscOrderSetting (
  orderSettingID INT(11) NOT NULL AUTO_INCREMENT,
  orderID INT(11) NOT NULL,
  code VARCHAR(100) NOT NULL DEFAULT '',
  name VARCHAR(255) DEFAULT NULL,
  description VARCHAR(255) DEFAULT NULL,
  type VARCHAR(100) DEFAULT NULL,
  valueType VARCHAR(16) NOT NULL,
  integerValue INT(11) DEFAULT NULL,
  smallValue VARCHAR(64) DEFAULT NULL,
  mediumValue VARCHAR(255) DEFAULT NULL,
  textValue TEXT,
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.order.OrderSettingBean',
  PRIMARY KEY  (orderSettingID),
  UNIQUE KEY orderSettingUKey (orderID, code),
  KEY orderIDIndex (orderID),
  FOREIGN KEY (`orderID`) REFERENCES `sscOrder` (`orderID`) ON DELETE CASCADE
);
