package com.softslate.commerce.administrator.customer;

import java.util.Arrays;
import java.util.Collection;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.customer.Customer;
import com.softslate.commerce.businessobjects.customer.CustomerAddress;
import com.softslate.commerce.businessobjects.customer.CustomerAddressProcessor;
import com.softslate.commerce.businessobjects.customer.CustomerProcessor;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class corresponding to requests to
 * "/administrator/CustomerAddressEditForm.do". Loads in the current values of
 * the record being edited and populates a
 * <code>CustomerAddressAddEditForm</code> object.
 * 
 * @author David Tobey
 * 
 */
public class CustomerAddressEditFormAction extends BaseAction {

	static Log log = LogFactory.getLog(CustomerAddressAddFormAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");
		CustomerAddressAddEditForm baseForm = (CustomerAddressAddEditForm) form;
		baseForm.initializeProperties(mapping, request);

		// Get the record to edit
		CustomerAddressProcessor processor = (CustomerAddressProcessor) baseForm
				.getBusinessObjectFactory().createObject(
						"customerAddressProcessorImplementer");
		CustomerAddress customerAddress = processor
				.loadCustomerAddressFromID(PropertyUtils.describe(form));
		BeanUtils.copyProperties(baseForm, customerAddress);

		// Load the parent if an id is present
		BusinessObjectFactory bof = (BusinessObjectFactory) request
				.getAttribute("businessObjectFactory");
		Customer customer = null;
		if (baseForm.getCustomerID() != null
				&& baseForm.getCustomerID().length() > 0) {
			CustomerProcessor parentPprocessor =  bof
					.createObject(CustomerProcessor.class);
			customer = parentPprocessor.loadCustomerFromID(PropertyUtils
					.describe(baseForm));
			request.setAttribute("customer", customer);
		}

		// Enforce role permissions
		if (!request.isUserInRole("superuser")
				&& !request.isUserInRole("administrator")
				&& !request.isUserInRole("customerAddressEdit")) {
			request.setAttribute("editPermission", "false");
		} else {
			request.setAttribute("editPermission", "true");
		}

		// Create breadcrumbing collection
		Collection bcs = Arrays.asList(new String[] { "Home", "Customers",
				"Customer Details", "Customer Delivery Addresses",
				"Customer Delivery Address Details" });
		request.setAttribute("breadcrumbs", bcs);

		// Create childSubNavigation collection
		/*Collection childsn = Arrays.asList(new String[] {
				"Customer Delivery Addresses", "Add Customer Delivery Address",
				"Customer Delivery Address Details" });
		request.setAttribute("childSubNavigation", childsn);*/

		request.setAttribute("customerAddress", customerAddress);
		request.setAttribute("customerAddressAddEditForm", baseForm);
		return mapping.findForward("success");
	}
}
