package com.softslate.commerce.administrator.customer;

import java.util.Arrays;
import java.util.Collection;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.businessobjects.customer.Customer;
import com.softslate.commerce.businessobjects.customer.CustomerProcessor;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class corresponding to requests to
 * "/administrator/CustomerEditForm.do". Loads in the current values of the
 * record being edited and populates a <code>CustomerAddEditForm</code>
 * object.
 * 
 * @author David Tobey
 * 
 */
public class CustomerEditFormAction extends BaseAction {

	static Log log = LogFactory.getLog(CustomerAddFormAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");
		CustomerAddEditForm baseForm = (CustomerAddEditForm) form;
		baseForm.initializeProperties(mapping, request);

		// Get the record to edit
		CustomerProcessor processor = (CustomerProcessor) baseForm
				.getBusinessObjectFactory().createObject(
						"customerProcessorImplementer");
		Customer customer = processor.loadCustomerFromID(PropertyUtils
				.describe(form));
		BeanUtils.copyProperties(baseForm, customer);

		// Enforce role permissions
		if (!request.isUserInRole("superuser")
				&& !request.isUserInRole("administrator")
				&& !request.isUserInRole("customerEdit")) {
			request.setAttribute("editPermission", "false");
		} else {
			request.setAttribute("editPermission", "true");
		}

		// Create breadcrumbing collection
		Collection bcs = Arrays.asList(new String[] { "Home", "Customers",
				"Customer Details" });
		request.setAttribute("breadcrumbs", bcs);

		request.setAttribute("customerAddEditForm", baseForm);
		request.setAttribute("customer", customer);
		return mapping.findForward("success");
	}
}
