package com.softslate.commerce.administrator.email;

import java.util.Calendar;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.email.Email;
import com.softslate.commerce.businessobjects.email.EmailProcessor;
import com.softslate.commerce.businessobjects.email.EmailSender;
import com.softslate.commerce.businessobjects.email.Template;
import com.softslate.commerce.businessobjects.email.TemplateParser;
import com.softslate.commerce.businessobjects.email.TemplateProcessor;
import com.softslate.commerce.businessobjects.order.Order;
import com.softslate.commerce.businessobjects.order.OrderProcessor;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class that corresponds to requests to
 * "/administrator/EmailAddFromTemplate.do". Processes a request to add a record
 * to the system.
 * 
 */
public class EmailAddFromTemplateAction extends BaseAction {

	static Log log = LogFactory.getLog(EmailAddFromTemplateAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form, HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");

		EmailAddEditForm baseForm = (EmailAddEditForm) form;

		// Validate that a templateID and orderNumber were submitted
		if (baseForm.getTemplateID() == null || baseForm.getOrderNumber() == null) {
			baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.required", "Template ID and order number"));
			saveErrors(baseForm.getRequest(), baseForm.getErrors());
			return mapping.findForward("success");
		}

		// Load the template and order
		Map parameters = PropertyUtils.describe(form);
		BusinessObjectFactory bof = baseForm.getBusinessObjectFactory();
		Order order = null;
		TemplateProcessor tp =  bof.createObject(TemplateProcessor.class);
		Template template = tp.loadTemplateFromID(parameters);
		OrderProcessor op =  bof.createObject(OrderProcessor.class);
		order = op.loadOrderDetails((Integer) parameters.get("orderNumber"));
		if (template == null || order == null) {
			baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.detail", "Unable to load template or order - no email created"));
			saveErrors(baseForm.getRequest(), baseForm.getErrors());
			return mapping.findForward("success");
		}

		// Create the Email object from the Template
		Email email = createEmailFromOrder(baseForm.getBusinessObjectFactory(), order,
				"ManuallyTriggered-" + template.getCode() + "-" + order.getOrderNumber() + "-"
						+ Calendar.getInstance().getTimeInMillis(), template);
		TemplateParser templateParser = createTemplateParser(bof, template, email, createExtraTemplateObjects(order));
		if (templateParser.parseEmail()) {
			email.setStatus("Unattempted");
			parameters.putAll(PropertyUtils.describe(email));
		} else {
			baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.detail", "Unable to parse template - no email created."));
			saveErrors(baseForm.getRequest(), baseForm.getErrors());
			return mapping.findForward("success");
		}

		// Save the Email to the database
		EmailProcessor processor = (EmailProcessor) baseForm.getBusinessObjectFactory().createObject(
				"emailProcessorImplementer");
		Map results = processor.addEmail(parameters);

		// Successful insert
		if (results == null || results.get("resultCode") == null || results.get("resultCode").equals("0")) {

			// Set the success messages
			baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("messages.recordAdded"));
			if (results != null && results.get("newRecord") != null) {
				Email newRecord = (Email) results.get("newRecord");

				// Set a request scope variable for sub classes to use
				request.setAttribute("newEmailID", new Integer(newRecord.getEmailID()));

				// Optionally send the Email
				if (baseForm.getAction() != null
						&& (baseForm.getAction().equals("Create And Send New Email") || baseForm.getAction().endsWith(
								"Send Email"))) {
					sendEmail(bof, newRecord);
					baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage("messages.detail", "Email Being Sent."));
				}
			}
			saveMessages(baseForm.getRequest(), baseForm.getMessages());
		}

		// Forward to the newly created email
		if (parameters.get("action") != null && !parameters.get("action").equals("") && results != null
				&& results.get("newRecord") != null) {
			Email newRecord = (Email) results.get("newRecord");
			ActionForward af = new ActionForward();
			af.setPath("/EmailEditForm.do?emailID=" + newRecord.getEmailID());
			af.setRedirect(false);
			return af;
		}
		return mapping.findForward("success");
	}

	protected Email createEmailFromOrder(BusinessObjectFactory bof, Order order, String emailType, Template template)
			throws Exception {
		Email email =  bof.createObject(Email.class);
		email.setCustomer(order.getCustomer());
		email.setOrder(order);
		email.setToEmails(order.getEmail1());
		email.setType(emailType);
		email.setTemplate(template);
		return email;
	}

	protected TemplateParser createTemplateParser(BusinessObjectFactory bof, Template template, Email email,
			Map<String, Object> extraTemplateObjects) throws Exception {
		TemplateParser templateParser =  bof.createObject(TemplateParser.class);
		templateParser.initialize(template, email, extraTemplateObjects);
		return templateParser;
	}

	protected void sendEmail(BusinessObjectFactory bof, Email email) throws Exception {
		EmailSender sender =  bof.createObject(EmailSender.class);
		sender.setEmail(email);
		sender.send();
	}

	protected Map<String, Object> createExtraTemplateObjects(Order order) throws Exception {
		return null;
	}
}
