package com.softslate.commerce.administrator.ordermanagement;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.product.Product;
import com.softslate.commerce.businessobjects.product.ProductProcessor;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Loads a product from its ID.
 * 
 * @author David Tobey
 */
public class LoadProductAction extends BaseAction {

	static Log log = LogFactory.getLog(LoadProductAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");

		ActionErrors errors = new ActionErrors();

		try {

			// Make sure a productID and quantity was submitted
			if (request.getParameter("orderProductCodes") == null
					|| request.getParameter("orderProductCodes").length() == 0) {
				errors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
						"errors.required", "Product selection"));
				saveErrors(request, (ActionMessages) errors);
				return mapping.findForward("failure");
			}
			if (request.getParameter("orderProductQuantities") == null
					|| request.getParameter("orderProductQuantities").length() == 0
					|| Integer.parseInt(request
							.getParameter("orderProductQuantities")) < 1) {
				errors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
						"errors.required", "Quantity"));
				saveErrors(request, (ActionMessages) errors);
				return mapping.findForward("failure");
			}

			// Get the product - same query as the customer side
			BusinessObjectFactory bof = (BusinessObjectFactory) request
					.getAttribute("businessObjectFactory");
			ProductProcessor productProcessor =  bof
					.createObject(ProductProcessor.class);
			Product product =  bof.createObject(Product.class);
			product.setCode(request.getParameter("orderProductCodes"));
			product = productProcessor.getProductFromCode(product);

			// Can't find the product, it's an error
			if (product.getProductID() == 0) {
				errors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
						"errors.productNotFound", product.getCode()));
				saveErrors(request, (ActionMessages) errors);
				return mapping.findForward("failure");
			}

			request.setAttribute("product", product);
			return mapping.findForward("success");

		} catch (Exception e) {
			if (log.isErrorEnabled())
				log.error("Exception follows: ", e);
			errors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
					"errors.detail", e.toString()));
			saveErrors(request, (ActionMessages) errors);
			return mapping.findForward("exception");
		}
	}
}
