package com.softslate.commerce.administrator.product;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.product.Discount;
import com.softslate.commerce.businessobjects.product.DiscountProcessor;
import com.softslate.commerce.customer.core.ActionUtils;
import com.softslate.commerce.customer.core.BaseAction;
import com.softslate.commerce.customer.product.ProductInitializer;

/**
 * Struts action class that corresponds to requests to
 * "/administrator/DiscountAdd.do". Processes a request to add a record to the
 * system.
 * 
 * @author David Tobey
 */
public class DiscountAddAction extends BaseAction {

	static Log log = LogFactory.getLog(DiscountAddAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");
		
		DiscountAddEditForm baseForm = (DiscountAddEditForm) form;

		// Handle generating the code from the name if it is empty
		ActionUtils actionUtils = (ActionUtils) baseForm
				.getBusinessObjectFactory().createUtilsObject(
						"actionUtilsImplementer");
		if ((baseForm.getCode() == null || baseForm.getCode().equals(""))
				&& (baseForm.getName() != null && !baseForm.getName()
						.equals(""))) {
			baseForm.setCode(actionUtils.generateCodeFromName(baseForm
					.getName()));
		}

		// Add the record
		DiscountProcessor processor = (DiscountProcessor) baseForm
				.getBusinessObjectFactory().createObject(
						"discountProcessorImplementer");
		Map results = processor.addDiscount(PropertyUtils.describe(form));

		// Successful insert
		if (results == null || results.get("resultCode") == null
				|| results.get("resultCode").equals("0")) {

			// Reload the store discounts in the application scope
			ProductInitializer pi = new ProductInitializer();
			pi.setServlet(servlet);
			pi.initializeStoreDiscounts(baseForm.getBusinessObjectFactory());

			// Set the success messages
			baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("messages.recordAdded"));
			if (results != null && results.get("newRecord") != null) {
				Discount newRecord = (Discount) results.get("newRecord");
				request.setAttribute("newDiscountRecord", newRecord);

				// entity.jsp wants the new record in the request scope.
				request.setAttribute("discount", newRecord);
				String params = "";
				if (request.getParameter("productID") != null) {
					params = params + "&productID="
							+ request.getParameter("productID");
				}
				if (request.getParameter("skuID") != null) {
					params = params + "&skuID=" + request.getParameter("skuID");
				}
				baseForm.getMessages().add(
						ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("messages.editNewRecord",
								"DiscountEditForm.do?discountID="
										+ newRecord.getDiscountID()));
				if (newRecord.getItemsAppliesType() != null
						&& newRecord.getItemsAppliesType().equals("[skus]")
						&& request.getParameter("productID") == null) {
					baseForm.getMessages().add(
							ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage("messages.linkedMessage",
									"SKU.do?assignments=all&parentIDField=discountID&discountID="
											+ newRecord.getDiscountID()
											+ params,
									"Assign SKUs to the new discount"));
				}
				if (newRecord.getRewardType() != null
						&& (newRecord.getRewardType().equals(
								"[rangeByQuantity]") || newRecord
								.getRewardType().equals("[rangeByPrice]"))) {
					baseForm.getMessages().add(
							ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage("messages.linkedMessage",
									"DiscountRange.do?discountID="
											+ newRecord.getDiscountID()
											+ params,
									"Create the ranges for the new discount"));
				}
			}
			saveMessages(baseForm.getRequest(), baseForm.getMessages());

			// If successful, signal the form to be reinstantiated.
			request.setAttribute("reset", "true");

			return mapping.findForward("success");
		}

		// Set the error messages
		if (results != null && results.get("existingRecord") != null) {
			Discount existingRecord = (Discount) results.get("existingRecord");
			String existingCode = existingRecord.getCode();
			baseForm.getErrors().add(
					ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.resultCode"
							+ results.get("resultCode"), results
							.get("duplicateField"), existingCode));
			baseForm.getErrors().add(
					ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.editExistingRecord",
							"DiscountEditForm.do?discountID="
									+ existingRecord.getDiscountID(), results
									.get("duplicateField"), existingCode));
			saveErrors(baseForm.getRequest(),
					(ActionMessages) baseForm.getErrors());
		}

		return mapping.findForward("success");
	}
}
