package com.softslate.commerce.administrator.product;

import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.businessobjects.product.AttributeProcessor;
import com.softslate.commerce.businessobjects.product.Product;
import com.softslate.commerce.businessobjects.product.ProductProcessor;
import com.softslate.commerce.businessobjects.product.SKU;
import com.softslate.commerce.businessobjects.product.SKUProcessor;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class corresponding to requests to
 * "/administrator/SKUEditForm.do". Loads in the current values of the record
 * being edited and populates a <code>SKUAddEditForm</code> object.
 * 
 * @author David Tobey
 */
public class SKUEditFormAction extends BaseAction {

	static Log log = LogFactory.getLog(SKUEditFormAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");
		SKUAddEditForm baseForm = (SKUAddEditForm) form;
		baseForm.initializeProperties(mapping, request);

		// Get the record to edit
		SKUProcessor processor = (SKUProcessor) baseForm
				.getBusinessObjectFactory().createObject(
						"skuProcessorImplementer");
		SKU sku = null;
		if (baseForm.getSkuID() != null) {
			sku = processor.loadSKUFromID(PropertyUtils.describe(form));
		}
		if (sku == null && baseForm.getSkuNumber() != null) {
			sku = processor.loadSKUFromNumber(PropertyUtils.describe(form));
		}
		BeanUtils.copyProperties(baseForm, sku);
		baseForm.mergeAttributeOptions();

		// Load the parent if an id is present
		Product product = null;
		if (baseForm.getProductID() != null && baseForm.getProductID().length() > 0
				&& !baseForm.getProductID().equals("0")) {
			ProductProcessor productProcessor = (ProductProcessor) baseForm
					.getBusinessObjectFactory().createObject(
							"productProcessorImplementer");
			Map parameters = new HashMap();
			parameters.put("productID", baseForm.getProductID());
			product = productProcessor.loadProductFromID(parameters);
			request.setAttribute("product", product);
		}

		// Load attributes for the attribute/option drop down.
		Collection attributes = null;
		if (baseForm.getProductID() != null
				&& baseForm.getProductID().length() > 0
				&& !baseForm.getProductID().equals("0")) {
			ProductProcessor productProcessor = (ProductProcessor) baseForm
					.getBusinessObjectFactory().createObject(
							"productProcessorImplementer");
			attributes = productProcessor.getAllAttributesAndOptions(Integer
					.parseInt(baseForm.getProductID()));
		} else {
			AttributeProcessor attributeProcessor = (AttributeProcessor) baseForm
					.getBusinessObjectFactory().createObject(
							"attributeProcessorImplementer");
			attributes = attributeProcessor.loadAllAttributes();
		}
		request.setAttribute("attributes", attributes);

		// Load products for the product drop down.
		ProductProcessor productProcessor = (ProductProcessor) baseForm
				.getBusinessObjectFactory().createObject(
						"productProcessorImplementer");
		Collection productNamesCodesIDs = productProcessor
				.loadAllProductNamesCodesIDs();
		request.setAttribute("productNamesCodesIDs", productNamesCodesIDs);

		// Enforce role permissions
		if (!request.isUserInRole("superuser")
				&& !request.isUserInRole("administrator")
				&& !request.isUserInRole("skuEdit")) {
			request.setAttribute("editPermission", "false");
		} else {
			request.setAttribute("editPermission", "true");
		}

		// Navigation
		if (request.getParameter("parentIDField") != null
				&& request.getParameter("parentIDField").equals("productID")) {

			// Create breadcrumbing collection
			Collection bcs = Arrays.asList(new String[] { "Home", "Products",
					"Product Details", "SKUs", "SKU Details" });
			request.setAttribute("breadcrumbs", bcs);
		} else {

			// Create breadcrumbing collection
			Collection bcs = Arrays.asList(new String[] { "Home", "SKUs",
					"SKU Details" });
			request.setAttribute("breadcrumbs", bcs);

		}

		request.setAttribute("sku", sku);
		request.setAttribute("skuAddEditForm", baseForm);
		return mapping.findForward("success");
	}
}
