package com.softslate.commerce.businessobjects.administrator;

import java.io.Serializable;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import com.softslate.commerce.businessobjects.core.BaseBusinessObject;

/**
 * Object representing an administrator role within the system.
 * <code>AdministratorRoleBean</code> is the default implementation of the
 * <code>AdministratorRole</code> interface for the application.
 * <p>
 * The database stores administrator roles in the
 * <code>sscAdministratorRole</code> table, and this class can be used to
 * represent a single row of that table. The Hibernate configuration file
 * <code>AdministratorRoleBean.hbm.xml</code> maps that table's columns to the
 * properties in this object.
 * 
 * @author David Tobey
 */
@Audited
@AuditTable("AudAdministratorRole")
public class AdministratorRoleBean extends BaseBusinessObject implements
		AdministratorRole, Serializable {

	private static final long serialVersionUID = -4218340642189037055L;

	static Log log = LogFactory.getLog(AdministratorRoleBean.class);

	private int administratorRoleID = 0;

	public int getAdministratorRoleID() {
		return administratorRoleID;
	}

	public void setAdministratorRoleID(int administratorRoleID) {
		this.administratorRoleID = administratorRoleID;
	}

	private int administratorID = 0;

	public int getAdministratorID() {
		return (this.administratorID);
	}

	public void setAdministratorID(int administratorID) {
		this.administratorID = administratorID;
	}

	private int roleID = 0;

	public int getRoleID() {
		return roleID;
	}

	public void setRoleID(int roleID) {
		this.roleID = roleID;
	}

	Administrator administrator = null;

	public Administrator getAdministrator() {
		return administrator;
	}

	public void setAdministrator(Administrator administrator) {
		this.administrator = administrator;
	}

	Role role = null;

	public Role getRole() {
		return role;
	}

	public void setRole(Role role) {
		this.role = role;
	}

}
