package com.softslate.commerce.businessobjects.administrator;

import java.util.ArrayList;
import java.util.Map;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.core.BaseBusinessProcessor;
import com.softslate.commerce.businessobjects.core.Settings;
import com.softslate.commerce.daos.core.UpgradesDAOBasic;

/**
 * Processes an upgrade for the application from one version to another.
 * 
 * @author David Tobey
 * 
 */
public class UpgradesProcessor extends BaseBusinessProcessor {

	static Log log = LogFactory.getLog(UpgradesProcessor.class);

	/**
	 * The path in the application to the directory containing upgrade scripts
	 * to be run against the database. By default,
	 * "/WEB-INF/classes/resources/upgrades".
	 */
	private String pathToUpgrades = null;

	public String getPathToUpgrades() {
		return pathToUpgrades;
	}

	public void setPathToUpgrades(String pathToUpgrades) {
		this.pathToUpgrades = pathToUpgrades;
	}

	private ArrayList upgrades = null;

	public ArrayList getUpgrades() {
		return upgrades;
	}

	public void setUpgrades(ArrayList upgrades) {
		this.upgrades = upgrades;
	}

	/**
	 * Processes an upgrade for the application from one version to another.
	 * Calls
	 * {@link com.softslate.commerce.daos.core.UpgradesDAOBasic#processUpgrades(ArrayList, String, Settings)}
	 * to execute the upgrade in the database.
	 * 
	 * @param parameters
	 *            A <code>Map</code> containing parameters for the upgrade
	 *            process.
	 * @return A <code>Settings</code> representing the application settins
	 *         after the upgrade has been processed.
	 * @throws Exception
	 */
	public Settings processUpgrades(Map parameters) throws Exception {
		if (parameters != null) {
			BeanUtils.copyProperties(this, parameters);
		}
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName()
					+ ": Running upgrades: " + getUpgrades());
		Settings settings = (Settings) getBusinessObjectFactory().createObject(
				"settingsImplementer");
		UpgradesDAOBasic upgradesDAO = (UpgradesDAOBasic) getDaoFactory()
				.createDAOFromClassName(
						"com.softslate.commerce.daos.core.UpgradesDAOBasic");
		return upgradesDAO.processUpgrades(getUpgrades(), getPathToUpgrades(),
				settings);
	}

	public void processSingleSqlScript(String pathToScript) throws Exception {
		log.info("Processing sql script: " + pathToScript);
		UpgradesDAOBasic upgradesDAO = (UpgradesDAOBasic) getDaoFactory().createDAOFromClassName(
				"com.softslate.commerce.daos.core.UpgradesDAOBasic");
		upgradesDAO.processSingleSqlScript(pathToScript);
	}
}
