package com.softslate.commerce.businessobjects.core;

import java.io.Serializable;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

/**
 * Represents a single setting used by the application. <code>SettingBean</code>
 * is the default implementation of the <code>Setting</code> interface for the
 * application.
 * <p>
 * The various settings represent everything from the default colors and fonts
 * used by the display templates, to logical settings defining the behavior of
 * various parts of the application.
 * <p>
 * The database stores settings in the <code>sscSetting</code> table, and this
 * class can be used to represent a single row of that table. The Hibernate
 * configuration file <code>SettingBean.hbm.xml</code> maps that table's
 * columns to the properties in this object.
 * 
 * @author David Tobey
 */
@Audited
@AuditTable("AudSetting")
public class SettingBean extends BaseBusinessProcessor implements Setting,
		Serializable {

	private static final long serialVersionUID = 4050487794147407160L;

	static Log log = LogFactory.getLog(SettingBean.class);

	/**
	 * The settingID for this <code>SettingBean</code>, when used as an
	 * interface to the <code>sscSetting</code> database table. Corresponds to
	 * <code>sscSetting.settingID</code>.
	 */
	private int settingID;

	public int getSettingID() {
		return (this.settingID);
	}

	public void setSettingID(int settingID) {
		this.settingID = settingID;
	}

	/**
	 * The code for this <code>SettingBean</code>, when used as an interface
	 * to the <code>sscSetting</code> database table. Corresponds to
	 * <code>sscSetting.code</code>.
	 */
	private String code = null;

	public String getCode() {
		return (this.code);
	}

	public void setCode(String code) {
		this.code = code;
	}

	/**
	 * The name for this <code>SettingBean</code>, when used as an interface
	 * to the <code>sscSetting</code> database table. Corresponds to
	 * <code>sscSetting.name</code>.
	 */
	private String name = null;

	public String getName() {
		return (this.name);
	}

	public void setName(String name) {
		this.name = name;
	}

	/**
	 * The description for this <code>SettingBean</code>, when used as an
	 * interface to the <code>sscSetting</code> database table. Corresponds to
	 * <code>sscSetting.description</code>.
	 */
	private String description = null;

	public String getDescription() {
		return (this.description);
	}

	public void setDescription(String description) {
		this.description = description;
	}

	/**
	 * The type for this <code>SettingBean</code>, when used as an interface
	 * to the <code>sscSetting</code> database table. Corresponds to
	 * <code>sscSetting.type</code>.
	 */
	private String type = null;

	public String getType() {
		return (this.type);
	}

	public void setType(String type) {
		this.type = type;
	}

	/**
	 * The value type for this <code>SettingBean</code>, when used as an
	 * interface to the <code>sscSetting</code> database table. Corresponds to
	 * <code>sscSetting.valueType</code>.
	 */
	private String valueType = null;

	public String getValueType() {
		return (this.valueType);
	}

	public void setValueType(String valueType) {
		this.valueType = valueType;
	}

	/**
	 * The integerValue for this <code>SettingBean</code>, when used as an
	 * interface to the <code>sscSetting</code> database table. Corresponds to
	 * <code>sscSetting.integerValue</code>.
	 */
	private Integer integerValue = null;

	public Integer getIntegerValue() {
		return (this.integerValue);
	}

	public void setIntegerValue(Integer integerValue) {
		this.integerValue = integerValue;
	}

	/**
	 * The smallValue for this <code>SettingBean</code>, when used as an
	 * interface to the <code>sscSetting</code> database table. Corresponds to
	 * <code>sscSetting.smallValue</code>.
	 */
	private String smallValue = null;

	public String getSmallValue() {
		return (this.smallValue);
	}

	public void setSmallValue(String smallValue) {
		this.smallValue = smallValue;
	}

	/**
	 * The mediumValue for this <code>SettingBean</code>, when used as an
	 * interface to the <code>sscSetting</code> database table. Corresponds to
	 * <code>sscSetting.mediumValue</code>.
	 */
	private String mediumValue = null;

	public String getMediumValue() {
		return (this.mediumValue);
	}

	public void setMediumValue(String mediumValue) {
		this.mediumValue = mediumValue;
	}

	/**
	 * The textValue for this <code>SettingBean</code>, when used as an
	 * interface to the <code>sscSetting</code> database table. Corresponds to
	 * <code>sscSetting.textValue</code>.
	 */
	private String textValue = null;

	public String getTextValue() {
		return (this.textValue);
	}

	public void setTextValue(String textValue) {
		this.textValue = textValue;
	}

}
