package com.softslate.commerce.businessobjects.customer;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.UUID;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.core.ApplicationException;
import com.softslate.commerce.businessobjects.core.BaseBusinessProcessor;
import com.softslate.commerce.daos.customer.CustomerAddressDAO;
import com.softslate.commerce.daos.customer.CustomerAddressGatewayDAO;
import com.softslate.commerce.daos.customer.CustomerGatewayDAO;

/**
 * Processes business logic having to do with a customer's delivery address.
 * Created in the Struts layer in response to requests for viewing, editing,
 * adding, and deleting delivery addresses.
 * <p>
 * <code>BasicCustomerAddressProcessor</code> is the default implementation of
 * the <code>CustomerAddressProcessor</code> interface for the application.
 * 
 * @author David Tobey
 * 
 */
public class BasicCustomerAddressProcessor extends BaseBusinessProcessor
		implements CustomerAddressProcessor {

	static Log log = LogFactory.getLog(BasicCustomerAddressProcessor.class);

	public Map loadCustomerAddressesAndCount(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName()
					+ ": Loading delivery addresses and count");
		CustomerAddressGatewayDAO gatewayDAO =  getDaoFactory()
				.createDAO(CustomerAddressGatewayDAO.class);
		return gatewayDAO.loadCustomerAddressesAndCount(parameters);
	}

	public Map addCustomerAddress(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			String code = (String) parameters.get("address1");
			log.info(getUser().getAdministrator().getUserName()
					+ ": Adding delivery address: " + code);
		}
		Map results = new HashMap();
		CustomerAddress customerAddress =  getBusinessObjectFactory()
				.createObject(CustomerAddress.class);
		BeanUtils.copyProperties(customerAddress, parameters);
		Customer customer = (Customer) getBusinessObjectFactory().createObject(
				"customerImplementer");
		customer.setCustomerID(customerAddress.getCustomerID());
		customerAddress.setCustomer(customer);
		CustomerAddressDAO dao =  getDaoFactory()
				.createDAO(CustomerAddressDAO.class);
		dao.setCustomerAddress(customerAddress);
		dao.insertCustomerAddress(true);
		results.put("newRecord", dao.getCustomerAddress());
		return results;
	}

	public CustomerAddress loadCustomerAddressFromID(Map parameters)
			throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName()
					+ ": Loading delivery address from id");
		CustomerAddress customerAddress =  getBusinessObjectFactory()
				.createObject(CustomerAddress.class);
		CustomerAddressDAO dao =  getDaoFactory()
				.createDAO(CustomerAddressDAO.class);
		BeanUtils.copyProperties(customerAddress, parameters);
		dao.setCustomerAddress(customerAddress);
		dao.loadCustomerAddressFromID();
		return dao.getCustomerAddress();
	}

	public Map editCustomerAddress(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			String code = (String) parameters.get("address1");
			log.info(getUser().getAdministrator().getUserName()
					+ ": Editing delivery address: " + code);
		}
		Map results = new HashMap();
		CustomerAddress customerAddress = loadCustomerAddressFromID(parameters);
		BeanUtils.copyProperties(customerAddress, parameters);
		Customer customer = (Customer) getBusinessObjectFactory().createObject(
				"customerImplementer");
		customer.setCustomerID(customerAddress.getCustomerID());
		customerAddress.setCustomer(customer);
		CustomerAddressDAO dao =  getDaoFactory()
				.createDAO(CustomerAddressDAO.class);
		dao.setCustomerAddress(customerAddress);
		dao.updateCustomerAddress(true);
		results.put("updatedRecord", dao.getCustomerAddress());
		return results;
	}

	public Map editDeleteCustomerAddresses(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName() + ": Editing/deleting multiple delivery addresses");
		CustomerAddressGatewayDAO dao = (CustomerAddressGatewayDAO) getDaoFactory().createDAO(
				"customerAddressGatewayDAOImplementer");
		Map results = dao.editDeleteCustomerAddresses(parameters, getBusinessObjectFactory());
		return results;
	}

	public void deleteCustomerAddress(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			String code = (String) parameters.get("address1");
			log.info(getUser().getAdministrator().getUserName()
					+ ": Deleting delivery address: " + code);
		}
		CustomerAddress customerAddress =  getBusinessObjectFactory()
				.createObject(CustomerAddress.class);
		BeanUtils.copyProperties(customerAddress, parameters);
		CustomerAddressDAO dao =  getDaoFactory()
				.createDAO(CustomerAddressDAO.class);
		dao.setCustomerAddress(customerAddress);
		dao.deleteCustomerAddress(true);
		return;
	}

	public CustomerAddress findMatchingAddressBookEntry(CustomerAddress customerAddress) throws Exception {
		CustomerAddress found = null;
		Collection<CustomerAddress> customerAddresses = (Collection<CustomerAddress>) getUser().getCustomer()
				.getCustomerAddressCollection();
		if (customerAddress != null && customerAddresses != null && customerAddresses.size() > 0) {
			for (CustomerAddress addr : customerAddresses) {
				// If it is the same reference return it.
				if (addr == customerAddress) {
					found = addr;
					break;
				}
				// Both billing or shipping
				if (customerAddress.getIsBilling() == addr.getIsBilling()) {
					String combined1 = StringUtils.stripToEmpty(customerAddress.getFirstName())
							+ StringUtils.stripToEmpty(customerAddress.getLastName())
							+ StringUtils.stripToEmpty(customerAddress.getAddress1())
							+ StringUtils.stripToEmpty(customerAddress.getAddress2())
							+ StringUtils.stripToEmpty(customerAddress.getCity())
							+ StringUtils.stripToEmpty(customerAddress.getState())
							+ StringUtils.stripToEmpty(customerAddress.getPostalCode());
					combined1 = combined1.replaceAll("\\s+", "").toLowerCase();

					String combined2 = StringUtils.stripToEmpty(addr.getFirstName())
							+ StringUtils.stripToEmpty(addr.getLastName())
							+ StringUtils.stripToEmpty(addr.getAddress1())
							+ StringUtils.stripToEmpty(addr.getAddress2()) + StringUtils.stripToEmpty(addr.getCity())
							+ StringUtils.stripToEmpty(addr.getState())
							+ StringUtils.stripToEmpty(addr.getPostalCode());
					combined2 = combined2.replaceAll("\\s+", "").toLowerCase();
					if (combined1.equals(combined2)) {
						found = addr;
						break;
					}
				}
			}
		}

		return found;
	}

	public Map addAddressBookAddress(Map parameters) throws Exception {
		return addAddressBookAddress(parameters, true);
	}

	public Map addAddressBookAddress(Map parameters, boolean updateDatabase) throws Exception {
		if (log.isInfoEnabled())
			log.info("Adding address and updating customer.");
		if (getUser().getCustomer() == null) {
			throw new ApplicationException(
					"User is not logged in; cannot update addresses.");
		}
		boolean addressExisted = false;
		// Save the current customer in case we have to revert to it
		Map existing = PropertyUtils.describe(getUser().getCustomer());
		Collection customerAddresses = getUser().getCustomer()
				.getCustomerAddressCollection();
		CustomerAddress customerAddress =  getBusinessObjectFactory()
				.createObject(CustomerAddress.class);
		if (customerAddresses == null) {
			getUser().getCustomer().setCustomerAddressCollection(
					new LinkedHashSet());
		}

		if (parameters != null) {
			BeanUtils.copyProperties(customerAddress, parameters);
		}

		CustomerAddress foundExistingAddress = findMatchingAddressBookEntry(customerAddress);
		if (foundExistingAddress != null) {
			foundExistingAddress.setIsDefault(true);
			customerAddress = foundExistingAddress;
			addressExisted = true;
		} else {
			customerAddress.setIsDefault(true);
			getUser().getCustomer().getCustomerAddressCollection().add(customerAddress);
			customerAddress.setCustomer(getUser().getCustomer());
		}

		// Set the others matching billing/shipping
		for (CustomerAddress addr : (Collection<CustomerAddress>) getUser()
				.getCustomer().getCustomerAddressCollection()) {
			if (addr != customerAddress && customerAddress.getIsBilling() == addr.getIsBilling()) {
				addr.setIsDefault(false);
			}
		}
		
		// Give this address a unique identifier in and return it
		UUID uuid = UUID.randomUUID();
		String randomUUIDString = uuid.toString();
		customerAddress.setIdentifier(randomUUIDString);
		
		
		// Update the database, this evicts so it can be skipped if more than
		// one may be added in the same request
		if (updateDatabase) {
			CustomerGatewayDAO customerGatewayDAO = (CustomerGatewayDAO) getDaoFactory().createDAO(
					"customerGatewayDAOImplementer");

			Map result = customerGatewayDAO.updateAddresses(getUser().getCustomer());
			// Revert to the previous info if any errors
			if (result != null && result.get("resultCode") != null && !result.get("resultCode").equals("0")) {
				BeanUtils.copyProperties(getUser().getCustomer(), existing);
			}
		}

		if (customerAddress != null){
			Map props = PropertyUtils.describe(customerAddress);
			if (addressExisted) {
				props.put("addressExisted", "1");
			}
			return props;
		} else {
			return null;
		}
	}

	public Map deleteAddressBookAddress(Map parameters) throws Exception {
		if (log.isInfoEnabled())
			log.info("deleting address and updating customer.");
		if (getUser().getCustomer() == null) {
			throw new ApplicationException(
					"User is not logged in; cannot delete addresses.");
		}
		// Save the current customer in case we have to revert to it
		Map existing = PropertyUtils.describe(getUser().getCustomer());

		Collection customerAddresses = getUser().getCustomer()
				.getCustomerAddressCollection();
		if (customerAddresses == null) {
			return null;
		}
		int customerAddressID = 0;
		try {
			customerAddressID = Integer.parseInt((String) parameters
					.get("customerAddressID"));
		} catch (Exception ex) {
			if (log.isInfoEnabled())
				log.info("Invalid customer address id to delete");
		}
		boolean deleted = false;
		boolean isDefault = false;
		boolean isBilling = false;
		Iterator<CustomerAddress> i = customerAddresses.iterator();
		while (i.hasNext()) {
			CustomerAddress addr = i.next();
			if (addr.getCustomerAddressID() == customerAddressID) {
				if (addr.getIsDefault()) {
					isDefault = true;
				}
				if (addr.getIsBilling()) {
					isBilling = true;
				}
				i.remove();
				deleted = true;
				break;
			}
		}
		if (deleted && isDefault) {
			int count = 0;
			for (CustomerAddress addr : (Collection<CustomerAddress>) customerAddresses) {
				if (addr.getIsBilling() == isBilling) {
					if (count == 0) {
						addr.setIsDefault(true);
					} else {
						addr.setIsDefault(false);
					}
					count++;
				}
			}
		}
		if (deleted) {
			// Update the database
			CustomerGatewayDAO customerGatewayDAO =  getDaoFactory()
					.createDAO(CustomerGatewayDAO.class);
			Map result = customerGatewayDAO.updateAddresses(getUser()
					.getCustomer());
			// Revert to the previous info if any errors
			if (result != null && result.get("resultCode") != null
					&& !result.get("resultCode").equals("0")) {
				BeanUtils.copyProperties(getUser().getCustomer(), existing);
			}
		}
		return null;
	}

	public Map updateAddressBookAddress(Map parameters) throws Exception {
		if (log.isInfoEnabled())
			log.info("Updating customer and address.");
		if (getUser().getCustomer() == null) {
			throw new ApplicationException(
					"User is not logged in; cannot update addresses.");
		}
		// Save the current customer in case we have to revert to it
		Map existing = PropertyUtils.describe(getUser().getCustomer());
		// Find the address
		Collection customerAddresses = getUser().getCustomer()
				.getCustomerAddressCollection();
		if (customerAddresses == null || parameters == null) {
			return null;
		}
		int customerAddressID = 0;
		try {
			customerAddressID = Integer.parseInt((String) parameters
					.get("customerAddressID"));
		} catch (Exception ex) {
			if (log.isInfoEnabled())
				log.info("Invalid customer address id to delete");
		}
		boolean updated = false;
		for (CustomerAddress addr : (Collection<CustomerAddress>) customerAddresses) {
			if (addr.getCustomerAddressID() == customerAddressID) {
				boolean wasDefault = addr.getIsDefault();
				BeanUtils.copyProperties(addr, parameters);
				// Keep the default value
				addr.setIsDefault(wasDefault);
				updated = true;
				break;
			}
		}
		if (updated) {
			// Update the database
			CustomerGatewayDAO customerGatewayDAO =  getDaoFactory()
					.createDAO(CustomerGatewayDAO.class);
			Map result = customerGatewayDAO.updateAddresses(getUser()
					.getCustomer());
			// Revert to the previous info if any errors
			if (result != null && result.get("resultCode") != null
					&& !result.get("resultCode").equals("0")) {
				BeanUtils.copyProperties(getUser().getCustomer(), existing);
			}
		}
		return null;
	}

	public CustomerAddress findCustomerAddress(
			Collection<CustomerAddress> customerAddresses, int customerAddressID) {
		if (customerAddresses == null || customerAddressID < 1) {
			return null;
		}
		for (CustomerAddress addr : (Collection<CustomerAddress>) customerAddresses) {
			if (addr.getCustomerAddressID() == customerAddressID) {
				return addr;
			}
		}
		return null;
	}

	public Map describeDeliveryOrBillingForAddressBook(Map parameters, String type, boolean removeType) {
		Map newMap = new HashMap();
		Iterator it = parameters.entrySet().iterator();
		while (it.hasNext()) {
			Map.Entry pairs = (Map.Entry) it.next();
			String key = (String) pairs.getKey();
			if (key.startsWith(type)) {
				String newName = key;
				if (removeType) {
					String newKey = key.replaceFirst(type, "");
					newName = newKey.substring(0, 1).toLowerCase() + newKey.substring(1);
				}
				newMap.put(newName, pairs.getValue());
			}
		}
		return newMap;

	}
}
