package com.softslate.commerce.businessobjects.customer;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import com.softslate.commerce.businessobjects.core.BusinessObject;

/**
 * Interface representing a customer's delivery address within the system.
 * application.
 * <p>
 * When the application needs to create an instance that implements
 * <code>CustomerAddress</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "customerAddressImplementer" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * The default "customerAddressImplementer" is
 * {@link com.softslate.commerce.businessobjects.customer.CustomerAddressBean}.
 * 
 * @author David Tobey
 * 
 */
@Audited
@AuditTable("AudCustomerAddress")
public interface CustomerAddress extends BusinessObject {
	abstract int getCustomerAddressID();

	abstract void setCustomerAddressID(int customerAddressID);

	abstract Customer getCustomer();

	abstract void setCustomer(Customer customer);

	abstract int getCustomerID();

	abstract void setCustomerID(int customerID);

	abstract String getStatus();

	abstract void setStatus(String status);

	abstract String getFirstName();

	abstract void setFirstName(String firstName);

	abstract String getLastName();

	abstract void setLastName(String lastName);

	abstract String getOrganization();

	abstract void setOrganization(String organization);

	abstract String getAddress1();

	abstract void setAddress1(String address1);

	abstract String getAddress2();

	abstract void setAddress2(String address2);

	abstract String getCity();

	abstract void setCity(String city);

	abstract String getState();

	abstract void setState(String state);

	abstract String getCountry();

	abstract void setCountry(String country);

	abstract String getPostalCode();

	abstract void setPostalCode(String postalCode);

	abstract String getPhone1();

	abstract void setPhone1(String phone1);

	abstract String getPhone2();

	abstract void setPhone2(String phone2);

	abstract String getEmail1();

	abstract void setEmail1(String email1);

	abstract String getEmail2();

	abstract void setEmail2(String email2);

	abstract String getExtra1();

	abstract void setExtra1(String extra1);

	abstract String getExtra2();

	abstract void setExtra2(String extra2);

	abstract String getExtra3();

	abstract void setExtra3(String extra3);

	abstract String getExtra4();

	abstract void setExtra4(String extra4);

	abstract String getExtra5();

	abstract void setExtra5(String extra5);

	abstract String getNotes();

	abstract void setNotes(String notes);

	abstract String getDeliveryFirstName();

	abstract void setDeliveryFirstName(String deliveryFirstName);

	abstract String getDeliveryLastName();

	abstract void setDeliveryLastName(String deliveryLastName);

	abstract String getDeliveryOrganization();

	abstract void setDeliveryOrganization(String deliveryOrganization);

	abstract String getDeliveryAddress1();

	abstract void setDeliveryAddress1(String deliveryAddress1);

	abstract String getDeliveryAddress2();

	abstract void setDeliveryAddress2(String deliveryAddress2);

	abstract String getDeliveryCity();

	abstract void setDeliveryCity(String deliveryCity);

	abstract String getDeliveryState();

	abstract void setDeliveryState(String deliveryState);

	abstract String getDeliveryCountry();

	abstract void setDeliveryCountry(String deliveryCountry);

	abstract String getDeliveryPostalCode();

	abstract void setDeliveryPostalCode(String deliveryPostalCode);

	abstract String getDeliveryPhone1();

	abstract void setDeliveryPhone1(String deliveryPhone1);

	abstract String getDeliveryPhone2();

	abstract void setDeliveryPhone2(String deliveryPhone2);

	abstract String getDeliveryEmail1();

	abstract void setDeliveryEmail1(String deliveryEmail1);

	abstract String getDeliveryEmail2();

	abstract void setDeliveryEmail2(String deliveryEmail2);

	abstract String getDeliveryExtra1();

	abstract void setDeliveryExtra1(String deliveryExtra1);

	abstract String getDeliveryExtra2();

	abstract void setDeliveryExtra2(String deliveryExtra2);

	abstract String getDeliveryExtra3();

	abstract void setDeliveryExtra3(String deliveryExtra3);

	abstract String getDeliveryExtra4();

	abstract void setDeliveryExtra4(String deliveryExtra4);

	abstract String getDeliveryExtra5();

	abstract void setDeliveryExtra5(String deliveryExtra5);

	abstract String getDeliveryNotes();

	abstract void setDeliveryNotes(String deliveryNotes);

	// Aliases to get/set if using as billing

	abstract String getBillingFirstName();

	abstract void setBillingFirstName(String billingFirstName);

	abstract String getBillingLastName();

	abstract void setBillingLastName(String billingLastName);

	abstract String getBillingOrganization();

	abstract void setBillingOrganization(String billingOrganization);

	abstract String getBillingAddress1();

	abstract void setBillingAddress1(String billingAddress1);

	abstract String getBillingAddress2();

	abstract void setBillingAddress2(String billingAddress2);

	abstract String getBillingCity();

	abstract void setBillingCity(String billingCity);

	abstract String getBillingState();

	abstract void setBillingState(String billingState);

	abstract String getBillingCountry();

	abstract void setBillingCountry(String billingCountry);

	abstract String getBillingPostalCode();

	abstract void setBillingPostalCode(String billingPostalCode);

	abstract String getBillingPhone1();

	abstract void setBillingPhone1(String billingPhone1);

	abstract String getBillingPhone2();

	abstract void setBillingPhone2(String billingPhone2);

	abstract String getBillingEmail1();

	abstract void setBillingEmail1(String billingEmail1);

	abstract String getBillingEmail2();

	abstract void setBillingEmail2(String billingEmail2);

	abstract String getBillingExtra1();

	abstract void setBillingExtra1(String billingExtra1);

	abstract String getBillingExtra2();

	abstract void setBillingExtra2(String billingExtra2);

	abstract String getBillingExtra3();

	abstract void setBillingExtra3(String billingExtra3);

	abstract String getBillingExtra4();

	abstract void setBillingExtra4(String billingExtra4);

	abstract String getBillingExtra5();

	abstract void setBillingExtra5(String billingExtra5);

	abstract String getBillingNotes();

	abstract void setBillingNotes(String billingNotes);

	abstract boolean getIsDefault();

	abstract void setIsDefault(boolean isDefault);

	abstract String getPaymentToken();

	abstract void setPaymentToken(String paymentToken);

	abstract String getCreditCardName();

	abstract void setCreditCardName(String creditCardName);

	abstract String getCreditCardType();

	abstract void setCreditCardType(String creditCardType);

	abstract String getCreditCardDate();

	abstract void setCreditCardDate(String creditCardDate);

	abstract String getCreditCardNumberDisplay();

	abstract void setCreditCardNumberDisplay(String creditCardNumberDisplay);

	abstract boolean getIsBilling();

	abstract void setIsBilling(boolean isBilling);
	
	abstract String getIdentifier();

	abstract void setIdentifier(String identifier);

}
