package com.softslate.commerce.businessobjects.email;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.core.BaseBusinessProcessor;
import com.softslate.commerce.businessobjects.customer.Customer;
import com.softslate.commerce.businessobjects.order.Order;
import com.softslate.commerce.daos.email.EmailDAO;
import com.softslate.commerce.daos.email.EmailGatewayDAO;

/**
 * @author Auto-Generated by SoftSlate Extender
 */
public class BasicEmailProcessor extends BaseBusinessProcessor implements EmailProcessor {

    static Log log = LogFactory.getLog(BasicEmailProcessor.class);

    public Map loadEmailsAndCount(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null)
            log.info(getUser().getAdministrator().getUserName() + ": Loading Emails and count");
        EmailGatewayDAO emailDAO =  getDaoFactory().createDAO(EmailGatewayDAO.class);
        return emailDAO.loadEmailsAndCount(parameters);
    }

    public Map addEmail(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null)
            log.info(getUser().getAdministrator().getUserName() + ": Adding Email");
        Map results = new HashMap();
        Email email =  getBusinessObjectFactory().createObject(Email.class);
        BeanUtils.copyProperties(email, parameters);
        if (email.getOrderID() != null && email.getOrderID().intValue() != 0) {
	        Order order =  getBusinessObjectFactory().createObject(Order.class);
	        order.setOrderID(email.getOrderID().intValue());
	        email.setOrder(order);
	    }
        if (email.getCustomerID() != null && email.getCustomerID().intValue() != 0) {
	        Customer customer =  getBusinessObjectFactory().createObject(Customer.class);
	        customer.setCustomerID(email.getCustomerID().intValue());
	        email.setCustomer(customer);
	    }
		if (email.getTemplateID() != null && email.getTemplateID().intValue() != 0) {
			Template template =  getBusinessObjectFactory().createObject(Template.class);
			template.setTemplateID(email.getTemplateID().intValue());
			email.setTemplate(template);
		}
        EmailDAO emailDAO =  getDaoFactory().createDAO(EmailDAO.class);
        emailDAO.setEmail(email);
        emailDAO.insertEmail(true);
        results.put("newRecord", emailDAO.getEmail());
        return results;
    }

    public Email loadEmailFromID(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null)
            log.info(getUser().getAdministrator().getUserName() + ": Loading Email from id");
        Email email =  getBusinessObjectFactory().createObject(Email.class);
        EmailDAO emailDAO =  getDaoFactory().createDAO(EmailDAO.class);
        BeanUtils.copyProperties(email, parameters);
        emailDAO.setEmail(email);
        emailDAO.loadEmailFromID();
        return emailDAO.getEmail();
    }

    public Map editEmail(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null)
            log.info(getUser().getAdministrator().getUserName() + ": Editing Email");
        Email email = loadEmailFromID(parameters);
        BeanUtils.copyProperties(email, parameters);
        if (email.getOrderID() != null && email.getOrderID().intValue() != 0) {
	        Order order =  getBusinessObjectFactory().createObject(Order.class);
	        order.setOrderID(email.getOrderID().intValue());
	        email.setOrder(order);
	    } else {
	        email.setOrder(null);
	    }
        if (email.getCustomerID() != null && email.getCustomerID().intValue() != 0) {
	        Customer customer =  getBusinessObjectFactory().createObject(Customer.class);
	        customer.setCustomerID(email.getCustomerID().intValue());
	        email.setCustomer(customer);
	    } else {
	        email.setCustomer(null);
	    }
		if (email.getTemplateID() != null && email.getTemplateID().intValue() != 0) {
			Template template =  getBusinessObjectFactory().createObject(Template.class);
			template.setTemplateID(email.getTemplateID().intValue());
			email.setTemplate(template);
		} else {
			email.setTemplate(null);
		}
        EmailDAO emailDAO =  getDaoFactory()
                .createDAO(EmailDAO.class);
        emailDAO.setEmail(email);
        emailDAO.updateEmail(true);
        Map results = new HashMap();
        results.put("updatedRecord", emailDAO.getEmail());
        return results;
    }

    public void editDeleteEmails(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null)
            log.info(getUser().getAdministrator().getUserName() + ": Editing/deleting multiple Emails");
		String[] idValues = (String[]) parameters.get("idValues");
		String[] displayFields = (String[]) parameters.get("displayFields");
		Order[] order = new Order[idValues.length];
		for (int i = 0; i < idValues.length; i++) {
			order[i] = null;
			for (int j = 0; j < displayFields.length; j++) {
				String displayField = displayFields[j];
				if (displayField.equals("orderID") || displayField.equals("order")) {
					displayFields[j] = "order";
					String[] fieldValues = (String[]) parameters.get("orderID");
					if (fieldValues != null && fieldValues[i] != null && !fieldValues[i].equals("0") && !fieldValues[i].equals("")) {
						if (log.isDebugEnabled())
							log.debug("Creating Order of " + fieldValues[i] + " for id: " + idValues[i]);
						Order object =  getBusinessObjectFactory().createObject(Order.class);
						object.setOrderID((new Integer(fieldValues[i])).intValue());
						order[i] = object;
					}
				}
			}
		}
		parameters.put("order", order);
		Customer[] customer = new Customer[idValues.length];
		for (int i = 0; i < idValues.length; i++) {
			customer[i] = null;
			for (int j = 0; j < displayFields.length; j++) {
				String displayField = displayFields[j];
				if (displayField.equals("customerID") || displayField.equals("customer")) {
					displayFields[j] = "customer";
					String[] fieldValues = (String[]) parameters.get("customerID");
					if (fieldValues != null && fieldValues[i] != null && !fieldValues[i].equals("0") && !fieldValues[i].equals("")) {
						if (log.isDebugEnabled())
							log.debug("Creating Customer of " + fieldValues[i] + " for id: " + idValues[i]);
						Customer object =  getBusinessObjectFactory().createObject(Customer.class);
						object.setCustomerID((new Integer(fieldValues[i])).intValue());
						customer[i] = object;
					}
				}
			}
		}
		parameters.put("customer", customer);
		Template[] template = new Template[idValues.length];
		for (int i = 0; i < idValues.length; i++) {
			template[i] = null;
			for (int j = 0; j < displayFields.length; j++) {
				String displayField = displayFields[j];
				if (displayField.equals("templateID") || displayField.equals("template")) {
					displayFields[j] = "template";
					String[] fieldValues = (String[]) parameters.get("templateID");
					if (fieldValues != null && fieldValues[i] != null && !fieldValues[i].equals("0")
							&& !fieldValues[i].equals("")) {
						if (log.isDebugEnabled())
							log.debug("Creating Template of " + fieldValues[i] + " for id: " + idValues[i]);
						Template object =  getBusinessObjectFactory().createObject(Template.class);
						object.setTemplateID((new Integer(fieldValues[i])).intValue());
						template[i] = object;
					}
				}
			}
		}
		parameters.put("template", template);
        EmailGatewayDAO emailDAO =  getDaoFactory().createDAO(EmailGatewayDAO.class);
        emailDAO.editDeleteEmails(parameters, getBusinessObjectFactory());
        return;
    }

    public void deleteEmail(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null)
            log.info(getUser().getAdministrator().getUserName() + ": Deleting Email");
        Email email =  getBusinessObjectFactory().createObject(Email.class);
        BeanUtils.copyProperties(email, parameters);
        EmailDAO emailDAO =  getDaoFactory().createDAO(EmailDAO.class);
        emailDAO.setEmail(email);
        emailDAO.deleteEmail(true);
        return;
    }

	public Collection<Email> loadAllEmails() throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName() + ": Loading all Emails");
		EmailGatewayDAO gatewayDAO =  getDaoFactory().createDAO(EmailGatewayDAO.class);
		return gatewayDAO.loadAllEmails();
	}

	public Integer emailsCountByTypeAndOrderID(String type, int orderID) throws Exception {
		EmailGatewayDAO gatewayDAO =  getDaoFactory().createDAO(EmailGatewayDAO.class);
		return gatewayDAO.emailsCountByTypeAndOrderID(type, orderID);
	}

	public Integer emailsCountByTypeAndCustomerID(String type, int customerID) throws Exception {
		EmailGatewayDAO gatewayDAO =  getDaoFactory().createDAO(EmailGatewayDAO.class);
		return gatewayDAO.emailsCountByTypeAndCustomerID(type, customerID);
	}

	public Integer emailsCountByTypeAndEmailAddress(String type, String emailAddress) throws Exception {
		EmailGatewayDAO gatewayDAO =  getDaoFactory().createDAO(EmailGatewayDAO.class);
		return gatewayDAO.emailsCountByTypeAndEmailAddress(type, emailAddress);
	}

	public Integer emailsCountByTypeAndEmailAddressAfterCreatedTime(String type, String emailAddress, String createdTime)
			throws Exception {
		EmailGatewayDAO gatewayDAO =  getDaoFactory().createDAO(EmailGatewayDAO.class);
		return gatewayDAO.emailsCountByTypeAndEmailAddressAfterCreatedTime(type, emailAddress, createdTime);
	}
}
