package com.softslate.commerce.businessobjects.order;

import java.util.Map;

import com.softslate.commerce.businessobjects.core.BusinessProcessor;

/**
 * Interface for business logic methods having to do with states. Created in the
 * Struts layer in response to requests for viewing, editing, adding, and
 * deleting states.
 * <p>
 * When the application needs to create an instance that implements
 * <code>StateProcessor</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "stateProcessorImplementer" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * The default "stateProcessorImplementer" is
 * {@link com.softslate.commerce.businessobjects.order.StateProcessor}.
 * 
 * @author David Tobey
 * 
 */
public interface StateProcessor extends BusinessProcessor {

	abstract Map loadStatesAndCount(Map parameters) throws Exception;

	abstract Map addState(Map parameters) throws Exception;

	abstract State loadStateFromID(Map parameters) throws Exception;

	abstract Map editState(Map parameters) throws Exception;

	abstract Map editDeleteStates(Map parameters) throws Exception;

	abstract void deleteState(Map parameters) throws Exception;
}
