package com.softslate.commerce.businessobjects.product;

import java.io.Serializable;
import java.util.Collection;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import com.softslate.commerce.businessobjects.core.BaseBusinessObject;

/**
 * Object representing a attribute within the system. <code>AttributeBean</code>
 * is the default implementation of the <code>Attribute</code> interface for
 * the application.
 * <p>
 * The database stores attributes in the <code>sscAttribute</code> table, and
 * this interface can be used to represent a single row of that table. The
 * Hibernate configuration file <code>AttributeBean.hbm.xml</code> maps that
 * table's columns to the properties in this object.
 * 
 * @author David Tobey
 * 
 */
@Audited
@AuditTable("AudAttribute")
public class AttributeBean extends BaseBusinessObject implements Attribute,
		Serializable {

	private static final long serialVersionUID = 8331660809744992245L;

	private int attributeID = 0;

	private String attributeCode = null;

	private String attributeName = null;

	private String attributeDescription = null;

	private boolean attributeIsActive = true;

	private String attributeType = null;

	private Double attributeUnitCost = null;

	private Double attributeUnitPrice = null;

	private Double attributeAltPrice = null;

	private Double attributeWeight = null;

	private String attributeSmallImage = null;

	private String attributeLargeImage = null;

	private Integer defaultOptionID = null;

	private boolean attributeIsRequired = false;

	private int attributeOrder = 0;

	public int getAttributeID() {
		return attributeID;
	}

	public void setAttributeID(int attributeID) {
		this.attributeID = attributeID;
	}

	public Double getAttributeAltPrice() {
		return attributeAltPrice;
	}

	public void setAttributeAltPrice(Double attributeAltPrice) {
		this.attributeAltPrice = attributeAltPrice;
	}

	public String getAttributeCode() {
		return attributeCode;
	}

	public void setAttributeCode(String attributeCode) {
		this.attributeCode = attributeCode;
	}

	public String getAttributeDescription() {
		return attributeDescription;
	}

	public void setAttributeDescription(String attributeDescription) {
		this.attributeDescription = attributeDescription;
	}

	public boolean getAttributeIsActive() {
		return attributeIsActive;
	}

	public void setAttributeIsActive(boolean attributeIsActive) {
		this.attributeIsActive = attributeIsActive;
	}

	public boolean getAttributeIsRequired() {
		return attributeIsRequired;
	}

	public void setAttributeIsRequired(boolean attributeIsRequired) {
		this.attributeIsRequired = attributeIsRequired;
	}

	public String getAttributeLargeImage() {
		return attributeLargeImage;
	}

	public void setAttributeLargeImage(String attributeLargeImage) {
		this.attributeLargeImage = attributeLargeImage;
	}

	public String getAttributeName() {
		return attributeName;
	}

	public void setAttributeName(String attributeName) {
		this.attributeName = attributeName;
	}

	public int getAttributeOrder() {
		return attributeOrder;
	}

	public void setAttributeOrder(int attributeOrder) {
		this.attributeOrder = attributeOrder;
	}

	public String getAttributeSmallImage() {
		return attributeSmallImage;
	}

	public void setAttributeSmallImage(String attributeSmallImage) {
		this.attributeSmallImage = attributeSmallImage;
	}

	public String getAttributeType() {
		return attributeType;
	}

	public void setAttributeType(String attributeType) {
		this.attributeType = attributeType;
	}

	public Double getAttributeUnitCost() {
		return attributeUnitCost;
	}

	public void setAttributeUnitCost(Double attributeUnitCost) {
		this.attributeUnitCost = attributeUnitCost;
	}

	public Double getAttributeUnitPrice() {
		return attributeUnitPrice;
	}

	public void setAttributeUnitPrice(Double attributeUnitPrice) {
		this.attributeUnitPrice = attributeUnitPrice;
	}

	public Double getAttributeWeight() {
		return attributeWeight;
	}

	public void setAttributeWeight(Double attributeWeight) {
		this.attributeWeight = attributeWeight;
	}

	public Integer getDefaultOptionID() {
		return defaultOptionID;
	}

	public void setDefaultOptionID(Integer defaultOptionID) {
		this.defaultOptionID = defaultOptionID;
	}

	private Collection productAttributes = null;

	public Collection getProductAttributes() {
		return (this.productAttributes);
	}

	public void setProductAttributes(Collection productAttributes) {
		this.productAttributes = productAttributes;
	}

	private Collection attributeOptions = null;

	public Collection getAttributeOptions() {
		return (this.attributeOptions);
	}

	public void setAttributeOptions(Collection attributeOptions) {
		this.attributeOptions = attributeOptions;
	}

	private Option defaultOption = null;

	public Option getDefaultOption() {
		return (this.defaultOption);
	}

	public void setDefaultOption(Option defaultOption) {
		this.defaultOption = defaultOption;
	}
}
