package com.softslate.commerce.businessobjects.product;

import java.util.Collection;
import java.util.Map;

import com.softslate.commerce.businessobjects.core.BusinessProcessor;

/**
 * Interface for business logic methods having to do with manufacturers. Created
 * in the Struts layer in response to requests for viewing, editing, adding, and
 * deleting manufacturers.
 * <p>
 * When the application needs to create an instance that implements
 * <code>ManufacturerProcessor</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "manufacturerProcessorImplementer" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * The default "manufacturerProcessorImplementer" is
 * {@link com.softslate.commerce.businessobjects.product.ManufacturerProcessor}.
 * 
 * @author David Tobey
 * 
 */
public interface ManufacturerProcessor extends BusinessProcessor {
	abstract Collection loadAllManufacturers() throws Exception;

	abstract Collection loadActiveManufacturers() throws Exception;

	abstract Map loadManufacturersAndCount(Map parameters) throws Exception;

	abstract Map addManufacturer(Map parameters) throws Exception;

	abstract Manufacturer loadManufacturerFromID(Map parameters)
			throws Exception;

	abstract Manufacturer loadManufacturerFromCode(Map parameters) throws Exception;
	
	abstract Manufacturer getManufacturerFromCode(Manufacturer manufacturer)
			throws Exception;

	abstract Manufacturer getManufacturerFromCode(Manufacturer manufacturer,
			boolean loadProducts) throws Exception;

	abstract Map editManufacturer(Map parameters) throws Exception;

	abstract Map editDeleteManufacturers(Map parameters) throws Exception;

	abstract void deleteManufacturer(Map parameters) throws Exception;
}
