package com.softslate.commerce.businessobjects.product;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import com.softslate.commerce.businessobjects.core.BusinessObject;

/**
 * Interface representing an attribute under a product in the system.
 * <p>
 * When the application needs to create an instance that implements
 * <code>ProductAttribute</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "productAttributeImplementer" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * The default "productAttributeImplementer" is
 * {@link com.softslate.commerce.businessobjects.product.ProductAttributeBean}.
 * <p>
 * Why is this a separate entity in the domain model? Please see
 * {@link com.softslate.commerce.businessobjects.product.ProductCategoryBean}
 * 
 * @author David Tobey
 * 
 * @see com.softslate.commerce.businessobjects.product.ProductCategory
 */
@Audited
@AuditTable("AudProductAttribute")
public interface ProductAttribute extends BusinessObject {
	abstract int getProductAttributeID();

	abstract void setProductAttributeID(int productAttributeID);

	abstract int getAttributeID();

	abstract void setAttributeID(int attributeID);

	abstract int getProductID();

	abstract void setProductID(int productID);

	abstract int getProductAttributeOrder();

	abstract void setProductAttributeOrder(int productAttributeOrder);

	abstract Product getProduct();

	abstract void setProduct(Product product);

	abstract Attribute getAttribute();

	abstract void setAttribute(Attribute attribute);

}
