package com.softslate.commerce.businessobjects.product;

import java.io.Serializable;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import com.softslate.commerce.businessobjects.core.BaseBusinessObject;

/**
 * Object representing a related product in the system.
 * <code>RelatedProductBean</code> is the default implementation of the
 * <code>RelatedProduct</code> interface for the application.
 * <p>
 * The database stores related products in the <code>sscRelatedProduct</code>
 * table, and this interface can be used to represent a single row of that
 * table. The Hibernate configuration file
 * <code>RelatedProductBean.hbm.xml</code> maps that table's columns to the
 * properties in this object.
 * 
 * @author David Tobey
 */
@Audited
@AuditTable("AudRelatedProduct")
public class RelatedProductBean extends BaseBusinessObject implements
		RelatedProduct, Serializable {

	private static final long serialVersionUID = 8845872714397175728L;

	static Log log = LogFactory.getLog(RelatedProductBean.class);

	private int relatedProductID = 0;

	private int parentProductID = 0;

	private int productID = 0;

	private int relatedProductOrder = 0;

	private String relatedType = null;

	private Product parentProduct;

	private Product product;

	public int getParentProductID() {
		return parentProductID;
	}

	public void setParentProductID(int parentProductID) {
		this.parentProductID = parentProductID;
	}

	public int getProductID() {
		return productID;
	}

	public void setProductID(int productID) {
		this.productID = productID;
	}

	public int getRelatedProductID() {
		return relatedProductID;
	}

	public void setRelatedProductID(int relatedProductID) {
		this.relatedProductID = relatedProductID;
	}

	public int getRelatedProductOrder() {
		return relatedProductOrder;
	}

	public void setRelatedProductOrder(int relatedProductOrder) {
		this.relatedProductOrder = relatedProductOrder;
	}

	public Product getParentProduct() {
		return parentProduct;
	}

	public void setParentProduct(Product parentProduct) {
		this.parentProduct = parentProduct;
	}

	public Product getProduct() {
		return product;
	}

	public void setProduct(Product product) {
		this.product = product;
	}

	public String getRelatedType() {
		return relatedType;
	}

	public void setRelatedType(String relatedType) {
		this.relatedType = relatedType;
	}
}
