package com.softslate.commerce.businessobjects.product;

import java.util.Collection;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import com.softslate.commerce.businessobjects.core.BusinessObject;

/**
 * Interface representing a sku within the system. A sku corresponds to a
 * {@link com.softslate.commerce.businessobjects.product.Product}, a
 * <code>Product</code> and any combination of
 * {@link com.softslate.commerce.businessobjects.product.Attribute}s and
 * {@link com.softslate.commerce.businessobjects.product.Option}s, or an
 * <code>Attribute</code> and its <code>Option</code>s, independent of any
 * products. It is used for tracking inventory and can it also be associated
 * with discounts. (SKU stands for "stock keeping unit".)
 * <p>
 * When the application needs to create an instance that implements
 * <code>sscSKU</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the "skuImplementer"
 * setting in the <code>appComponents.properties</code> file.
 * <p>
 * The default "skuImplementer" is
 * {@link com.softslate.commerce.businessobjects.product.SKUBean}.
 * 
 * @author David Tobey
 */
@Audited
@AuditTable("AudSKU")
public interface SKU extends BusinessObject {
	abstract int getSKUID();

	abstract void setSKUID(int skuID);

	abstract Integer getAttributeID1();

	abstract void setAttributeID1(Integer attributeID1);

	abstract Integer getAttributeID2();

	abstract void setAttributeID2(Integer attributeID2);

	abstract Integer getAttributeID3();

	abstract void setAttributeID3(Integer attributeID3);

	abstract Integer getAttributeID4();

	abstract void setAttributeID4(Integer attributeID4);

	abstract Integer getAttributeID5();

	abstract void setAttributeID5(Integer attributeID5);

	abstract int getInventoryLevel();

	abstract void setInventoryLevel(int inventoryLevel);

	abstract String getNotes();

	abstract void setNotes(String notes);

	abstract Integer getOptionID1();

	abstract void setOptionID1(Integer optionID1);

	abstract Integer getOptionID2();

	abstract void setOptionID2(Integer optionID2);

	abstract Integer getOptionID3();

	abstract void setOptionID3(Integer optionID3);

	abstract Integer getOptionID4();

	abstract void setOptionID4(Integer optionID4);

	abstract Integer getOptionID5();

	abstract void setOptionID5(Integer optionID5);

	abstract Integer getProductID();

	abstract void setProductID(Integer productID);

	abstract int getSkuID();

	abstract void setSkuID(int skuID);

	abstract String getSkuNumber();

	abstract void setSkuNumber(String skuNumber);

	abstract boolean getTrackInventory();

	abstract void setTrackInventory(boolean trackInventory);

	abstract Product getProduct();

	abstract void setProduct(Product product);

	abstract String getExtra1();

	abstract void setExtra1(String extra1);

	abstract String getExtra2();

	abstract void setExtra2(String extra2);

	abstract String getExtra3();

	abstract void setExtra3(String extra3);

	abstract String getSkuName();

	abstract void setSkuName(String skuName);

	abstract String getSkuMessage();

	abstract void setSkuMessage(String skuMessage);

	Collection getSkuDiscounts();

	void setSkuDiscounts(Collection skuDiscounts);
	
	Collection getSkuShippingRules();

	void setSkuShippingRules(Collection skuShippingRules);

}
