package com.softslate.commerce.businessobjects.tax;

import java.util.Map;

import com.softslate.commerce.businessobjects.core.BusinessObject;

/**
 * Interface for objects that are able to process tax charges in the store.
 * Custom tax processors must simply implement <code>TaxProcessor</code>.
 * <p>
 * To change or add to the list of tax processors currently active in the
 * system, modify the "activeTaxProcessors" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * During checkout, the Struts layer invokes each of the "activeTaxProcessors"
 * by calling their <code>processTax(Map parameters)</code> method to process
 * tax on the current user's cart.
 * <p>
 * The default "activeTaxProcessor" is
 * {@link com.softslate.commerce.businessobjects.tax.BasicTaxProcessor}.
 * 
 * @author David Tobey
 * 
 */
public interface TaxProcessor extends BusinessObject {
	/**
	 * Process the tax charges for the current user's order. The incoming
	 * <code>Map</code> should contain any input parameters from the user
	 * related to how the tax should be charged.
	 * 
	 * @param parameters
	 *            A <code>Map</code> containing any input parameters from the
	 *            user related to how the tax should be charged.
	 * @return A Map containing a least one key named "resultCode", which
	 *         contains a String parsable into an integer, representing the
	 *         results of the processing. 0 for the resultCode indicates
	 *         success, anything else indicates an error and that processing
	 *         should stop.
	 * @throws Exception
	 */
	abstract Map processTax(Map parameters) throws Exception;
}
