package com.softslate.commerce.customer.customer;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.customer.core.BaseAction;
import com.softslate.commerce.customer.core.BaseForm;
import com.softslate.commerce.businessobjects.order.CartDiscountProcessor;

/**
 * Struts action class used for requests to log out of a customer account. Used
 * for requests to "/Logout.do".
 * <p>
 * The <code>execute</code> method of this class nullifies the
 * <code>customer</code> property of the
 * {@link com.softslate.commerce.businessobjects.core.User} instance in the
 * current user's session.
 * 
 * @author David Tobey
 * 
 */
public class LogoutAction extends BaseAction {
	static Log log = LogFactory.getLog(LogoutAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isInfoEnabled())
			log.info("Starting action.");
		BaseForm baseForm = new BaseForm();
		baseForm.setServlet(getServlet());
		baseForm.initializeProperties(mapping, request);
		// If the customer object in the user object is null, then we have not
		// logged in.
		if (log.isDebugEnabled())
			log.debug("Logging out the customer");
		synchronized (request.getSession()) {
			if (baseForm.getUser().getCustomer() != null) {

				// Nullify the session's customer object
				baseForm.getUser().setCustomer(null);

				// Reset the loggedIn request variable
				request.setAttribute("loggedIn", null);

				// Force shipping and taxes to be recalculated
				baseForm.getRequest().getSession().setAttribute(
						"shippingAndTaxesProcessed", "false");

				// Recalculate discounts since they may depend on being logged
				// in
				CartDiscountProcessor dp = (CartDiscountProcessor) baseForm
						.getBusinessObjectFactory().createObject(
								"cartDiscountProcessorImplementer");
				dp.processCustomerLogout(null);

				// Clear the cart if called for in the settings
				String clearCart = (String) baseForm.getSettings().getValue(
						"clearCartOnLogout");
				if (null != baseForm.getUser()
						&& null != baseForm.getUser().getOrder()
						&& null != clearCart && clearCart.equals("1")) {
					baseForm.getUser().setOrder(null);
				}

				// Delete the autoLoginToken cookie so they don't log right back
				// in
				Cookie[] cookies = request.getCookies();
				if (cookies != null) {
					for (int i = 0; i < cookies.length; i++) {
						if (cookies[i] != null && cookies[i].getName() != null
								&& cookies[i].getName().equals("sscaltoken")) {
							Cookie autoLoginCookie = cookies[i];
							autoLoginCookie.setMaxAge(0);
							response.addCookie(autoLoginCookie);
						}
						if (null != clearCart && clearCart.equals("1")
								&& cookies[i] != null
								&& cookies[i].getName() != null
								&& cookies[i].getName().equals("sscsctoken")) {
							Cookie savedCartCookie = cookies[i];
							savedCartCookie.setMaxAge(0);
							response.addCookie(savedCartCookie);
							request.setAttribute("skipLoadSavedCart", "true");
						}
					}
				}

				// Tell the CustomerRequestProcessor to skip the auto login as
				// we forward to another action.
				request.setAttribute("skipAutoLogin", "true");

				baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("messages.loggedOut"));
				saveMessages(baseForm.getRequest(), baseForm.getMessages());
			}
			return mapping.findForward("success");
		}
	}
}
