package com.softslate.commerce.customer.json;

import java.io.PrintWriter;

import javax.servlet.ServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.BodyTagSupport;
import javax.servlet.jsp.tagext.Tag;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.core.JsonExclusionStrategy;

/* Print the json result object */
public class PrintJsonResultTag extends BodyTagSupport {

	static Log log = LogFactory.getLog(PrintJsonResultTag.class);

	private static final long serialVersionUID = -3429125710874333443L;
	private PageContext pc = null;
	private Tag parent = null;

	public void setPageContext(PageContext p) {
		pc = p;
	}

	public void setParent(Tag t) {
		parent = t;
	}

	public Tag getParent() {
		return parent;
	}

	public int doStartTag() throws JspException {

		try {
			JsonResult result = (JsonResult) pc.getRequest().getAttribute("jsonResult");
			if (result == null) {
				result = new JsonResult();
				pc.getRequest().setAttribute("jsonResult", result);
			}
			PrintWriter out = pc.getResponse().getWriter();
			try {
				if (result.errors != null && result.errors.size() > 0) {
					result.success = false;
				} else {
					result.success = true;
				}
				log.info("Printing json result: " + result);
				JsonExclusionStrategy exclusionStrategy = initExclusionStrategy(pc.getRequest());
				Gson gson = new GsonBuilder().setPrettyPrinting().setExclusionStrategies(exclusionStrategy).create();
				String gsonResult = gson.toJson(result);
				out.println(gsonResult);
			} catch (Exception e) {
				throw e;
			} finally {
				out.flush();
			}
			return SKIP_BODY;

		} catch (Exception ex) {
			throw new JspTagException("An Exception occurred.");
		}

	}

	public int doEndTag() throws JspException {
		return EVAL_PAGE;
	}

	public void release() {
		pc = null;
		parent = null;
	}
	
	protected JsonExclusionStrategy initExclusionStrategy(ServletRequest request) throws Exception {
		BusinessObjectFactory bof = (BusinessObjectFactory) request.getAttribute("businessObjectFactory");
		return  bof.createObject(JsonExclusionStrategy.class);
	}

}