package com.softslate.commerce.customer.order;

import java.util.Collection;
import java.util.Iterator;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.customer.core.ActionUtils;
import com.softslate.commerce.businessobjects.order.CartProcessor;
import com.softslate.commerce.businessobjects.order.OrderItem;

/**
 * Struts action class for submissions to all items in the current user's cart.
 * Used for requests to "/CartEdit.do".
 * 
 * @author Jason McWilliams
 * 
 */
public class CartEditAction extends CartAddAction {

	static Log log = LogFactory.getLog(CartAddAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isInfoEnabled())
			log.info("Starting action.");
		CartEditForm baseForm = (CartEditForm) form;

		synchronized (request.getSession()) {

			// Check that the user's session has not expired.
			if (baseForm.getUser() == null
					|| baseForm.getUser().getOrder() == null) {
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.sessionExpired"));
			} else {
				if (baseForm.getOrderItemIDs() != null
						&& baseForm.getOrderItemIDs().length > 0) {
					for (String itemID : baseForm.getOrderItemIDs()) {
						int orderItemID = Integer.parseInt(itemID);
						// Try to find the OrderItem
						if (baseForm.getUser().getOrder().firstDelivery() != null
								&& baseForm.getUser().getOrder()
										.firstDelivery()
										.getOrderItemCollection() != null) {
							Collection ois = baseForm.getUser().getOrder()
									.firstDelivery().getOrderItemCollection();
							Iterator i = ois.iterator();
							while (i.hasNext()) {
								OrderItem oi = (OrderItem) i.next();
								if (oi.getOrderItemID() == orderItemID) {
									baseForm.getOrderItems().add(oi);
								}
							}
						}
					}
				}
				// Error if the OrderItem can't be found
				if (baseForm.getOrderItems() == null
						|| baseForm.getOrderAttributes().size() == 0) {
					baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage("errors.orderItemNotFound"));
					saveErrors(baseForm.getRequest(), (ActionMessages) baseForm
							.getErrors());
					return baseForm.getMapping().findForward("failure");
				}
			}

			// Process the editing of the item
			baseForm.getRequest().getSession().setAttribute(
					"shippingAndTaxesProcessed", "false");
			CartProcessor cartProcessor = (CartProcessor) baseForm
					.getBusinessObjectFactory().createObject(
							"cartProcessorImplementer");
			Map results = cartProcessor.processEditItem(PropertyUtils
					.describe(baseForm));
			ActionUtils actionUtils = (ActionUtils) baseForm
					.getBusinessObjectFactory().createUtilsObject(
							"actionUtilsImplementer");
			String forward = actionUtils.processCartAddResults(results,
					baseForm);
			actionUtils.loadEstimatedShippingOptions(baseForm);
			return mapping.findForward(forward);
		}
	}

}
