package com.softslate.commerce.customer.order;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.order.OrderItem;
import java.util.Collection;
import java.util.ArrayList;
/**
 * Struts form class for submissions to edit all items in the current user's
 * cart. Used for requests to "/CartEdit.do".
 * <p>
 * The <code>validate</code> method of this class extends
 * {@link com.softslate.commerce.customer.order.CartAddForm#validate(ActionMapping, HttpServletRequest)}.
 * 
 * @author Jason McWilliams
 * 
 * @see com.softslate.commerce.customer.order.CartEditAction
 */
public class CartEditForm extends CartAddForm {

	/**
	 * 
	 */
	private static final long serialVersionUID = -3813326718845802669L;

	static Log log = LogFactory.getLog(CartEditForm.class);

	private String[] orderItemIDs = null;

	private Collection<OrderItem> orderItems = new ArrayList<OrderItem>();

	public Collection<OrderItem> getOrderItems() {
		return orderItems;
	}

	public void setOrderItems(Collection<OrderItem> orderItems) {
		this.orderItems = orderItems;
	}

	public String[] getOrderItemIDs() {
		return orderItemIDs;
	}

	public void setOrderItemIDs(String[] orderItemIDs) {
		this.orderItemIDs = orderItemIDs;
	}

	public ActionErrors validate(ActionMapping mapping,
			HttpServletRequest request) {
		if (log.isInfoEnabled())
			log.info("Starting validation.");
		initializeProperties(mapping, request);

		// Check for catalog mode
		try {
			String catalogMode = (String) getSettings().getValue("catalogMode");
			if (catalogMode.equals("1")) {
				getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.catalogMode"));
				return getErrors();
			}
		} catch (Exception e) {
		}

		// Check that at least one product code was submitted.
		validateProductCode();

		// Validate attributes
		validateAttributes();

		// Check order items
		if (getOrderItemIDs() == null || getOrderItemIDs().length == 0) {
			String message = getMessage("order", "cartItemEdit.item", null);
			getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.required", message));
		}

		if (log.isDebugEnabled())
			log.debug("Finished validation.");
		if (log.isInfoEnabled() && !getErrors().isEmpty())
			log.info("Validation error(s): " + getErrors().toString());
		return getErrors();
	}

}
