package com.softslate.commerce.customer.order;

import java.util.Collection;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.businessobjects.payment.PaymentProcessor;
import com.softslate.commerce.customer.core.ActionUtils;
import com.softslate.commerce.customer.core.BaseAction;
import com.softslate.commerce.customer.core.BaseForm;

/**
 * Struts action class that corresponds to the store's checkout link. Used for
 * requests to "/Checkout.do" and as a superclass for all of the other action
 * classes in the checkout process.
 * <p>
 * The <code>execute</code> method of this class decides what checkout screen
 * to direct the user to based on the status of the session and the store's
 * database configuration.
 * <p>
 * The idea behind <code>CheckoutAction</code> is to provide a single
 * superclass that manages nearly all of the forms and processing for the
 * checkout process. This allows developers to easily create custom checkout
 * screens by extending <code>CheckoutAction</code>, using its methods in
 * different combinations.
 * <p>
 * To allow developers to more easily override these methods, a delegate pattern
 * is used, in which most methods are delegated to CheckoutUtils and
 * CheckoutUtilsImpl, which may be subclassed to implement custom functionality.
 * 
 * @author David Tobey
 */
public class CheckoutAction extends BaseAction {
	static Log log = LogFactory.getLog(CheckoutAction.class);

	/**
	 * Forwards the user to the appropriate checkout screen based on the status
	 * of the session and the store's database settings.
	 */
	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isInfoEnabled())
			log.info("Starting action.");
		BaseForm baseForm = (BaseForm) new BaseForm();
		baseForm.setServlet(getServlet());
		baseForm.initializeProperties(mapping, request);

		ActionUtils actionUtils = (ActionUtils) baseForm
				.getBusinessObjectFactory().createUtilsObject(
						"actionUtilsImplementer");
		String checkoutScreen = actionUtils.findCheckoutScreen(baseForm);
		return baseForm.getMapping().findForward(checkoutScreen);
	}

	public void prepareLoginForm(BaseForm baseForm) throws Exception {
		CheckoutUtils checkoutUtils = (CheckoutUtils) baseForm
				.getBusinessObjectFactory().createUtilsObject(
						"checkoutUtilsImplementer");
		checkoutUtils.prepareLoginForm(baseForm, getServlet());
		return;
	}

	public void prepareRegisterForm(BaseForm baseForm) throws Exception {
		CheckoutUtils checkoutUtils = (CheckoutUtils) baseForm
				.getBusinessObjectFactory().createUtilsObject(
						"checkoutUtilsImplementer");
		checkoutUtils.prepareRegisterForm(baseForm, getServlet());
		return;
	}

	public ActionForward processCheckoutLogin(BaseForm baseForm,
			HttpServletResponse response) throws Exception {
		CheckoutUtils checkoutUtils = (CheckoutUtils) baseForm
				.getBusinessObjectFactory().createUtilsObject(
						"checkoutUtilsImplementer");
		return checkoutUtils.processCheckoutLogin(baseForm, response);
	}

	public ActionForward processCheckoutRegister(BaseForm baseForm)
			throws Exception {
		CheckoutUtils checkoutUtils = (CheckoutUtils) baseForm
				.getBusinessObjectFactory().createUtilsObject(
						"checkoutUtilsImplementer");
		return checkoutUtils.processCheckoutRegister(baseForm);
	}

	public ActionForward processDeclineLogin(BaseForm baseForm)
			throws Exception {
		CheckoutUtils checkoutUtils = (CheckoutUtils) baseForm
				.getBusinessObjectFactory().createUtilsObject(
						"checkoutUtilsImplementer");
		return checkoutUtils.processDeclineLogin(baseForm);
	}

	public Collection loadShippingOptions(BaseForm baseForm) throws Exception {
		CheckoutUtils checkoutUtils = (CheckoutUtils) baseForm
				.getBusinessObjectFactory().createUtilsObject(
						"checkoutUtilsImplementer");
		return checkoutUtils.loadShippingOptions(baseForm);
	}

	public void prepareCheckoutAddressesForm(CheckoutAddressesForm baseForm)
			throws Exception {
		CheckoutUtils checkoutUtils = (CheckoutUtils) baseForm
				.getBusinessObjectFactory().createUtilsObject(
						"checkoutUtilsImplementer");
		String allowAddressBook = (String) baseForm.getSettings().getValue(
				"allowAddressBook");
				
		// Do work using address books if user is logged in and it is turned on.
		if ("1".equals(allowAddressBook) && baseForm.getUser() != null
				&& baseForm.getUser().getCustomer() != null) {
			checkoutUtils.prepareCheckoutAddressesAddressBookForm(baseForm);
		} else {
			checkoutUtils.prepareCheckoutAddressesForm(baseForm);
		}

	}

	public void prepareCheckoutShippingForm(BaseForm baseForm) throws Exception {
		CheckoutUtils checkoutUtils = (CheckoutUtils) baseForm
				.getBusinessObjectFactory().createUtilsObject(
						"checkoutUtilsImplementer");
		checkoutUtils.prepareCheckoutShippingForm(baseForm);
	}

	public ActionForward processCheckoutAddresses(BaseForm baseForm)
			throws Exception {
		CheckoutUtils checkoutUtils = (CheckoutUtils) baseForm
				.getBusinessObjectFactory().createUtilsObject(
						"checkoutUtilsImplementer");
		return checkoutUtils.processCheckoutAddresses(baseForm);
	}

	public ActionForward processCheckoutAddresses(BaseForm baseForm,
			HttpServletResponse response) throws Exception {
		CheckoutUtils checkoutUtils = (CheckoutUtils) baseForm
				.getBusinessObjectFactory().createUtilsObject(
						"checkoutUtilsImplementer");
		return checkoutUtils.processCheckoutAddresses(baseForm, response);
	}

	public Map processInventoryDiscountCheck(BaseForm baseForm)
			throws Exception {
		CheckoutUtils checkoutUtils = (CheckoutUtils) baseForm
				.getBusinessObjectFactory().createUtilsObject(
						"checkoutUtilsImplementer");
		return checkoutUtils.processInventoryDiscountCheck(baseForm);
	}

	public ActionForward processCheckoutShipping(BaseForm baseForm)
			throws Exception {
		CheckoutUtils checkoutUtils = (CheckoutUtils) baseForm
				.getBusinessObjectFactory().createUtilsObject(
						"checkoutUtilsImplementer");
		return checkoutUtils.processCheckoutShipping(baseForm);
	}

	public Map processShipping(BaseForm baseForm) throws Exception {
		CheckoutUtils checkoutUtils = (CheckoutUtils) baseForm
				.getBusinessObjectFactory().createUtilsObject(
						"checkoutUtilsImplementer");
		return checkoutUtils.processShipping(baseForm);
	}

	public Map processTax(BaseForm baseForm) throws Exception {
		CheckoutUtils checkoutUtils = (CheckoutUtils) baseForm
				.getBusinessObjectFactory().createUtilsObject(
						"checkoutUtilsImplementer");
		return checkoutUtils.processTax(baseForm);
	}

	public void preparePaymentForms(BaseForm baseForm) throws Exception {
		CheckoutUtils checkoutUtils = (CheckoutUtils) baseForm
				.getBusinessObjectFactory().createUtilsObject(
						"checkoutUtilsImplementer");
		checkoutUtils.preparePaymentForms(baseForm, getServlet());
		return;
	}

	public void preparePaymentForm(ActionForm paymentForm, BaseForm baseForm)
			throws Exception {
		CheckoutUtils checkoutUtils = (CheckoutUtils) baseForm
				.getBusinessObjectFactory().createUtilsObject(
						"checkoutUtilsImplementer");
		checkoutUtils.preparePaymentForm(paymentForm, baseForm, getServlet());
		return;
	}

	public ActionForward processCheckoutPayment(BaseForm baseForm)
			throws Exception {
		CheckoutUtils checkoutUtils = (CheckoutUtils) baseForm
				.getBusinessObjectFactory().createUtilsObject(
						"checkoutUtilsImplementer");
		return checkoutUtils.processCheckoutPayment(baseForm, getServlet());
	}

	public boolean validatePaymentForm(ActionForm paymentForm, BaseForm baseForm)
			throws Exception {
		CheckoutUtils checkoutUtils = (CheckoutUtils) baseForm
				.getBusinessObjectFactory().createUtilsObject(
						"checkoutUtilsImplementer");
		return checkoutUtils.validatePaymentForm(paymentForm, baseForm);
	}

	public int processPayment(PaymentProcessor paymentProcessor,
			ActionForm paymentForm, BaseForm baseForm) throws Exception {
		CheckoutUtils checkoutUtils = (CheckoutUtils) baseForm
				.getBusinessObjectFactory().createUtilsObject(
						"checkoutUtilsImplementer");
		return checkoutUtils.processPayment(paymentProcessor, paymentForm,
				baseForm);
	}

	public ActionForward processCheckoutConfirm(BaseForm baseForm)
			throws Exception {
		CheckoutUtils checkoutUtils = (CheckoutUtils) baseForm
				.getBusinessObjectFactory().createUtilsObject(
						"checkoutUtilsImplementer");
		return checkoutUtils.processCheckoutConfirm(baseForm);
	}

	public Map processOrderComplete(BaseForm baseForm) throws Exception {
		CheckoutUtils checkoutUtils = (CheckoutUtils) baseForm
				.getBusinessObjectFactory().createUtilsObject(
						"checkoutUtilsImplementer");
		return checkoutUtils.processOrderComplete(baseForm);
	}
}
