package com.softslate.commerce.customer.order;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.customer.core.BaseForm;

/**
 * Struts form class for submissions to record a selected shipping option during
 * checkout. Used for requests to "/CheckoutShipping.do".
 * <p>
 * The <code>validate</code> method of this class checks to make sure a
 * shipping option was selected, if the "shippingRequired" setting is set for
 * the store.
 * 
 * @author David Tobey
 */
public class CheckoutShippingForm extends BaseForm {

	private static final long serialVersionUID = -7455174714123096572L;

	static Log log = LogFactory.getLog(CheckoutShippingForm.class);

	private boolean isSubmitted = false;

	public boolean isSubmitted() {
		return isSubmitted;
	}

	public void setIsSubmitted(boolean isSubmitted) {
		this.isSubmitted = isSubmitted;
	}

	/**
	 * The shipping method selected by the user during checkout. For the
	 * {@link com.softslate.commerce.businessobjects.shipping.BasicShippingProcessor},
	 * this field corresponds to the <code>code</code> property of the
	 * <code>sscShippingMethod</code> database table.
	 */
	private String shippingOption = null;

	public String getShippingOption() {
		return shippingOption;
	}

	public void setShippingOption(String string) {
		shippingOption = string;
	}

	public ActionErrors validate(ActionMapping mapping,
			HttpServletRequest request) {
		if (log.isInfoEnabled())
			log.info("Starting validation.");
		
		initializeProperties(mapping, request);
		
		// Ensure a shipping option was selected
		try {
			String shippingRequired = (String) getSettings().getValue(
					"shippingRequired");
			if (shippingRequired.equals("1")) {
				validateShippingOption();
			}
		} catch (Exception e) {
		}

		setIsSubmitted(true);

		if (log.isDebugEnabled())
			log.debug("Finished validation.");
		if (log.isInfoEnabled() && !getErrors().isEmpty())
			log.info("Validation error(s): " + getErrors().toString());
		return getErrors();
	}

	/**
	 * Ensures a the <code>shippingOption</code> property has been set.
	 */
	public void validateShippingOption() {
		if (getShippingOption() == null || getShippingOption().length() == 0) {
			String message = getMessage("order",
					"checkoutAddresses.shippingOption", null);
			getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.required", message));
		}
	}
}
