package com.softslate.commerce.customer.payment.cresecure;

import java.util.Collection;
import java.util.HashMap;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.customer.Customer;
import com.softslate.commerce.businessobjects.customer.CustomerAddress;
import com.softslate.commerce.businessobjects.order.Order;
import com.softslate.commerce.businessobjects.payment.cresecure.CRESecureProcessor;
import com.softslate.commerce.businessobjects.payment.cresecure.CRESecureValidationResult;
import com.softslate.commerce.customer.order.CheckoutAction;

/**
 * Will verify that the payment was successful, completes the order, and will
 * respond accordingly
 * 
 * @author Jason McWilliams
 */

public class CRESecureCompleteAction extends CheckoutAction {
	static Log log = LogFactory.getLog(CRESecureCompleteAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isInfoEnabled())
			log.info("Starting action.");

		CRESecureCompleteForm baseForm = (CRESecureCompleteForm) form;
		baseForm.initializeProperties(mapping, request);

		synchronized (baseForm.getRequest().getSession()) {

			// Check for a page refresh or session timeout
			if (baseForm.getUser().getOrder() == null) {
				if (log.isInfoEnabled())
					log.info("Session no longer exists");
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("messages.sessionTimeout"));
				saveErrors(baseForm.getRequest(),
						(ActionMessages) baseForm.getErrors());
				return baseForm.getMapping().findForward("failure");
			}
			String displayCheckoutConfirm = (String) baseForm.getSettings().getValue("displayCheckoutConfirm");

			if ("1".equals(displayCheckoutConfirm) || StringUtils.isNotBlank(baseForm.getTransId())) {
				String allowAddressBook = (String) baseForm.getSettings().getValue("allowAddressBook");
				Order order = baseForm.getUser().getOrder();
				Customer customer = baseForm.getUser().getCustomer();
				CRESecureProcessor creSecureProcessor = (CRESecureProcessor) baseForm.getBusinessObjectFactory()
						.createObject("creSecureProcessorImplementer");
				try {
					CRESecureValidationResult result = new CRESecureValidationResult();
					if (!"1".equals(displayCheckoutConfirm)) {
						result = creSecureProcessor.validateTransaction(order,baseForm.getTransId());
					}
					
					if ("1".equals(displayCheckoutConfirm) || result.getIsValid()) {

						// Set a session variable to check on next request for
						// the order it thank you page, let them view the
						// invoice as long as they have the cookie?
						baseForm.getRequest().getSession()
								.setAttribute("creorderid", Integer.toString(order.getOrderID()));

						// If we are using tokens and we have an address book
						// record, update it
						if ("1".equals(displayCheckoutConfirm) || StringUtils.isNotBlank(baseForm.getTokenId())) {
							order.setPaymentProfileID(baseForm.getTokenId());

							if ("1".equals(allowAddressBook)) {
								if (baseForm.getUser().getCustomMap() == null)
									baseForm.getUser().setCustomMap(new HashMap());
								int usingBillingAddressBookID = 0;
								try {
									usingBillingAddressBookID = (Integer) baseForm.getUser().getCustomMap()
											.get("usingBillingAddressBookID");
								} catch (Exception ex) {
									usingBillingAddressBookID = 0;
								}
								String usingBillingAddressBookUID = (String) baseForm.getUser().getCustomMap()
										.get("usingBillingAddressBookUID");

								if (customer != null && customer.getCustomerAddressCollection() != null) {
									boolean updated = false;

									// First look for matching token
									for (CustomerAddress addr : (Collection<CustomerAddress>) customer
											.getCustomerAddressCollection()) {
										if (StringUtils.isNotBlank(addr.getPaymentToken())
												&& baseForm.getTokenId().equals(addr.getPaymentToken())) {
											creSecureProcessor.updateCustomerAddressBook(addr.getCustomerAddressID(),
													baseForm);
											updated = true;
											break;
										}
									}

									// Next look for address id or UID in
									// addresses in case the address does not
									// have a token yet
									if (!updated && usingBillingAddressBookID > 0) {
										for (CustomerAddress addr : (Collection<CustomerAddress>) customer
												.getCustomerAddressCollection()) {
											if (addr.getCustomerAddressID() == usingBillingAddressBookID) {
												creSecureProcessor.updateCustomerAddressBook(
														addr.getCustomerAddressID(), baseForm);
												updated = true;
												break;
											}
										}
									}
									if (!updated && StringUtils.isNotBlank(usingBillingAddressBookUID)) {
										for (CustomerAddress addr : (Collection<CustomerAddress>) customer
												.getCustomerAddressCollection()) {
											if (StringUtils.isNotBlank(addr.getIdentifier())
													&& addr.getIdentifier().equals(usingBillingAddressBookUID)) {
												creSecureProcessor.updateCustomerAddressBook(
														addr.getCustomerAddressID(), baseForm);
												updated = true;
												break;
											}
										}
									}

								} // End Customer Address Book Block

							} // End Address Book Block


						} // End Token Exists Block (may not be using tokens)

						baseForm.getRequest().setAttribute("order", order);

						// We will store this information in the payment even if
						// its just storing the card for the first time. We
						// might want to get to view all of this information at
						// some point as it has a lot more than subsequent token
						// charges
						creSecureProcessor.process(baseForm);

						// Process Orde Complete
						if ("1".equals(displayCheckoutConfirm)) {
							// Do not complete the order, going to show confirm
						} else {
							processOrderComplete(baseForm);
						}

						// Set form and user in base to send emails
						request.setAttribute("baseForm", baseForm);
						request.setAttribute("user", baseForm.getUser());
						baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE,
								new ActionMessage("messages.detail", "VALID"));
						if (StringUtils.isNotBlank(baseForm.getCcNumber()) && baseForm.getCcNumber().startsWith("XXXX")
								&& baseForm.getCcNumber().length() == 16) {
							baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE,
									new ActionMessage("messages.detail", baseForm.getCcNumber()));
						}
						saveMessages(baseForm.getRequest(), (ActionMessages) baseForm.getMessages());
						if ("1".equals(displayCheckoutConfirm)) {
							return mapping.findForward("successNoEmail");
						} else {
							return mapping.findForward("success");
						}

					} else {
						baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
								new ActionMessage("errors.creUnableToValidatePayment"));
						saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
						return baseForm.getMapping().findForward("failure");
					}

				} catch (Exception ex) {
					if (log.isErrorEnabled()) {
						log.error("Exception validating payment. ", ex);
					}
					baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage("errors.creUnableToValidatePayment"));
					saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
					return baseForm.getMapping().findForward("failure");
				}

			} else {
				if (log.isErrorEnabled()) {
					log.error("No transaction ID was sent to validation action.");
				}
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.creUnableToValidatePayment"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
				return baseForm.getMapping().findForward("failure");
			}

		}
	}

}
