package com.softslate.commerce.customer.payment.paypoint;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;
import java.security.MessageDigest;

import com.softslate.commerce.customer.core.BaseForm;

/**
 * The form that handles the callback from Paypoint.
 * 
 * @author Jason McWilliams
 */
public class PaypointCallbackForm extends BaseForm {

	private static final long serialVersionUID = 198407271841461241L;

	static Log log = LogFactory.getLog(PaypointCallbackForm.class);

	private String amount = null;

	private String auth_code = null;

	private String code = null;

	private String currency = null;

	private String cv2avs = null;

	private String hash = null;

	private String message = null;

	private String resp_code = null;

	private String test_status = null;

	private String trans_id = null;

	// Only used when repeat is sent, this is currently not implemented

	private String expiry = null;

	private String card_no = null;

	private String customer = null;

	public String getAmount() {
		return amount;
	}

	public void setAmount(String amount) {
		this.amount = amount;
	}

	public String getAuth_code() {
		return auth_code;
	}

	public void setAuth_code(String auth_code) {
		this.auth_code = auth_code;
	}

	public String getCode() {
		return code;
	}

	public void setCode(String code) {
		this.code = code;
	}

	public String getCurrency() {
		return currency;
	}

	public void setCurrency(String currency) {
		this.currency = currency;
	}

	public String getCv2avs() {
		return cv2avs;
	}

	public void setCv2avs(String cv2avs) {
		this.cv2avs = cv2avs;
	}

	public String getHash() {
		return hash;
	}

	public void setHash(String hash) {
		this.hash = hash;
	}

	public String getMessage() {
		return message;
	}

	public void setMessage(String message) {
		this.message = message;
	}

	public String getResp_code() {
		return resp_code;
	}

	public void setResp_code(String resp_code) {
		this.resp_code = resp_code;
	}

	public String getTest_status() {
		return test_status;
	}

	public void setTest_status(String test_status) {
		this.test_status = test_status;
	}

	public String getTrans_id() {
		return trans_id;
	}

	public void setTrans_id(String trans_id) {
		this.trans_id = trans_id;
	}

	public String getExpiry() {
		return expiry;
	}

	public void setExpiry(String expiry) {
		this.expiry = expiry;
	}

	public String getCard_no() {
		return card_no;
	}

	public void setCard_no(String card_no) {
		this.card_no = card_no;
	}

	public String getCustomer() {
		return customer;
	}

	public void setCustomer(String customer) {
		this.customer = customer;
	}

	public ActionErrors validate(ActionMapping mapping,
			HttpServletRequest request) {
		if (log.isInfoEnabled())
			log.info("Starting validation.");
		initializeProperties(mapping, request);

		boolean error = false;

		if (getTrans_id() == null || getTrans_id().length() < 1) {
			error = true;
			String message = getMessage("payment",
					"payment.paypointTransIdRequired", null);
			getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.required", message));
		}

		// Check the hash, if we are checking the hash
		String paypointUseDigest = (String) getSettings().getValue(
				"paypointUseDigest");
		String paypointRemotePassword = (String) getSettings().getValue(
				"paypointRemotePassword");
		String paypointCallback = (String) getSettings().getValue(
				"paypointCallback");

		if (paypointUseDigest != null && paypointUseDigest.equals("1")
				&& !error) {

			if (paypointRemotePassword == null) {
				error = true;
				String message = getMessage("payment",
						"payment.paypointRemotePasswordRequired", null);
				getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.required", message));
			}
			if (paypointCallback == null) {
				error = true;
				String message = getMessage("payment",
						"payment.paypointCallbackRequired", null);
				getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.required", message));
			}
			if (getHash() == null || getHash().length() < 1) {
				error = true;
				String message = getMessage("payment",
						"payment.paypointHashRequired", null);
				getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.required", message));
			}
			if (!error) {
				try {
					String queryString = request.getQueryString();
					if (queryString == null) {
						queryString = "";
					}
					String newQueryString = removeHash(request.getQueryString());
					String stringToDigest = request.getRequestURI() + "?"
							+ newQueryString + paypointRemotePassword;
					MessageDigest messageDigest = java.security.MessageDigest
							.getInstance("MD5");

					messageDigest.update(stringToDigest.getBytes("UTF-8"));
					byte[] hash = messageDigest.digest();
					StringBuffer hexString = new StringBuffer();
					final String HEX_DIGITS = "0123456789abcdef";
					for (int i = 0; i < hash.length; i++) {
						int b = hash[i] & 0xFF;
						hexString.append(HEX_DIGITS.charAt(b >>> 4)).append(
								HEX_DIGITS.charAt(b & 0xF));
					}
					String hashString = hexString.toString();

					if (!getHash().equals(hashString)) {
						String message = getMessage("payment",
								"payment.paypointSecurityError", null);
						getErrors().add(ActionMessages.GLOBAL_MESSAGE,
								new ActionMessage("errors.detail", message));
					}
				} catch (Exception ex) {
					String message = getMessage("payment",
							"payment.paypointUnknownError", null);
					getErrors().add(ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage("errors.detail", message));
					if (log.isInfoEnabled())
						log.info("Paypoint Exception: ", ex);
				}
			}
		}

		if (log.isDebugEnabled())
			log.debug("Finished validation.");
		if (log.isInfoEnabled() && !getErrors().isEmpty())
			log.info("Validation error(s): " + getErrors().toString());

		return getErrors();
	}

	private String removeHash(String requestURL) {
		String removeFromQuery = "hash=" + getHash();
		String requestQuery = requestURL.replace(removeFromQuery, "");
		return requestQuery;

	}
}
