package com.softslate.commerce.daos.administrator;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.administrator.Administrator;
import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.core.User;
import com.softslate.commerce.daos.core.BaseAdminGatewayDAOHibernate;

/**
 * Generic DAO that runs miscellaneous queries for the administrator
 * application.
 * 
 * @author David Tobey
 */
public class AdministratorGatewayDAOHibernate extends
		BaseAdminGatewayDAOHibernate implements AdministratorGatewayDAO {

	static Log log = LogFactory.getLog(AdministratorGatewayDAOHibernate.class);

	public Map processLogin(User user) throws Exception {
		Map result = new HashMap();
		AdministratorDAO administratorDAO =  getDaoFactory()
				.createDAO(AdministratorDAO.class);
		administratorDAO.setAdministrator(user.getAdministrator());
		// Load the administrator
		administratorDAO.loadFromUserNameAndPassword();
		if (administratorDAO.getAdministrator() == null) {
			result.put("resultCode", "errors.invalidLogin");
			result.put("validAdministrator", null);
			return result;
		}
		administratorDAO.loadRoles();
		result.put("resultCode", "0");
		result.put("validAdministrator", administratorDAO.getAdministrator());
		return result;
	}

	public Map processLogin(String administratorUserName, String administratorPassword) throws Exception {
		Map result = new HashMap();
		AdministratorDAO administratorDAO =  getDaoFactory().createDAO(AdministratorDAO.class);
		// Load the administrator
		Administrator administrator = administratorDAO.loadFromUserNameAndPassword(administratorUserName,
				administratorPassword);
		if (administrator == null) {
			result.put("resultCode", "errors.invalidLogin");
			result.put("validAdministrator", null);
			return result;
		}
		administratorDAO.loadRoles(administrator);
		result.put("resultCode", "0");
		result.put("validAdministrator", administrator);
		return result;
	}

	public Administrator processApiLogin(String apiLoginID, String apiAuthenticationToken) throws Exception {
		AdministratorDAO administratorDAO =  getDaoFactory().createDAO(AdministratorDAO.class);
		Administrator administrator = administratorDAO.loadFromApiUserNameAndPassword(apiLoginID,
				apiAuthenticationToken);
		if (administrator != null) {
			administratorDAO.loadRoles();
			administratorDAO.getAdministrator().setLastLogin(formatDateTime(new Date()));
		}
		return administrator;
	}

	public Map loadAdministratorsAndCount(Map parameters) throws Exception {
		return loadItemsAndCount(parameters);
	}

	public Map editDeleteAdministrators(Map parameters,
			BusinessObjectFactory bof) throws Exception {
		return editDeleteItems(parameters, bof);
	}

}
