package com.softslate.commerce.daos.customer;

import java.util.Collection;
import java.util.List;
import java.util.Map;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.core.User;
import com.softslate.commerce.businessobjects.customer.Customer;
import com.softslate.commerce.businessobjects.customer.CustomerAddress;
import com.softslate.commerce.daos.core.DAOInterface;

/**
 * Interface for a DAO that runs miscellaneous queries related to customers.
 * 
 * @author David Tobey
 * 
 */
public interface CustomerGatewayDAO extends DAOInterface {
	abstract void updateCustomerAndAddress(Customer customer,
			CustomerAddress customerAddress) throws Exception;

	abstract void updateCustomerInsertAddress(Customer customer,
			CustomerAddress customerAddress) throws Exception;

	abstract Map processRegister(User user) throws Exception;

	public Map processLogin(User user) throws Exception;

	public Map processAutoLogin(User user) throws Exception;

	abstract Map updateAddresses(Customer customer) throws Exception;

	abstract Map updateCustomer(Customer customer, boolean forcePasword)
			throws Exception;

	abstract Map loadCustomersAndCount(Map parameters) throws Exception;

	abstract Map editDeleteCustomers(Map parameters, BusinessObjectFactory bof)
			throws Exception;

	abstract int loadMaxCustomerID() throws Exception;

	abstract Collection loadAllCustomersInRange(int firstID, int lastID)
			throws Exception;

	abstract Map refreshCustomerFromDatabase(User user) throws Exception;
			
	abstract Collection<Customer> loadCustomersWithNoCIMProfileByDate(
			String cutoffDate) throws Exception;
	
	abstract List<Object[]> loadCustomersWithNoCIMProfileByDate(
			String createdStart, String createdEnd) throws Exception;
}
