package com.softslate.commerce.daos.product;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;

import com.softslate.commerce.businessobjects.product.Discount;
import com.softslate.commerce.daos.core.BaseDAO;
import com.softslate.commerce.daos.core.DAOInterface;
import com.softslate.commerce.daos.core.DataAccessException;

/**
 * Generic DAO to manage the <code>sscDiscount</code> table of the database.
 * 
 * @author David Tobey
 */
public class DiscountDAOHibernate extends BaseDAO implements DAOInterface,
		DiscountDAO {

	static Log log = LogFactory.getLog(DiscountDAOHibernate.class);

	private Discount discount = null;

	public Discount getDiscount() {
		return (this.discount);
	}

	public void setDiscount(Discount discount) {
		this.discount = discount;
	}

	public void loadDiscountFromID() throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			Discount discount = (Discount) session.get(Discount.class,
					new Integer(getDiscount().getDiscountID()));
			if (discount != null) {
				PropertyUtils.copyProperties(getDiscount(), discount);
			}
			setDiscount(discount);
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public void loadDiscountFromCode() throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "discount.selectDiscountFromCode");
			q.setString("code", getDiscount().getCode());
			Discount loadedDiscount = (Discount) q.uniqueResult();
			setDiscount(loadedDiscount);
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public void insertDiscount() throws DataAccessException {
		insertDiscount(false);
	}

	public void insertDiscount(boolean commit) throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			session.save(getDiscount());
			if (commit) {
				getDaoFactory().commitTransaction();
			}
			evictCache();
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public void updateDiscount() throws DataAccessException {
		updateDiscount(false);
	}

	public void updateDiscount(boolean commit) throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			session.update(getDiscount());
			if (commit) {
				getDaoFactory().commitTransaction();
			}
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public void deleteDiscount() throws DataAccessException {
		deleteDiscount(false);
	}

	public void deleteDiscount(boolean commit) throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			setDiscount((Discount) session.load(Discount.class, new Integer(
					getDiscount().getDiscountID())));
			session.delete(getDiscount());
			if (commit) {
				getDaoFactory().commitTransaction();
			}
			evictCache();
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}
}
