package com.softslate.commerce.daos.shipping;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.Session;

import com.softslate.commerce.businessobjects.shipping.ShippingRate;
import com.softslate.commerce.daos.core.BaseDAO;
import com.softslate.commerce.daos.core.DAOInterface;
import com.softslate.commerce.daos.core.DataAccessException;

/**
 * Generic DAO to manage the <code>sscShippingRate</code> table of the
 * database.
 * 
 * @author David Tobey
 * 
 */
public class ShippingRateDAOHibernate extends BaseDAO implements DAOInterface,
        ShippingRateDAO {

    static Log log = LogFactory.getLog(ShippingRateDAOHibernate.class);

    private ShippingRate shippingRate = null;

    public ShippingRate getShippingRate() {
        return (this.shippingRate);
    }

    public void setShippingRate(ShippingRate shippingRate) {
        this.shippingRate = shippingRate;
    }

    public void loadShippingRateFromID() throws Exception {
        Session session = getDaoFactory().startSession();
        try {
            ShippingRate record = (ShippingRate) session.get(
                    ShippingRate.class, new Integer(getShippingRate()
                            .getShippingRateID()));
            PropertyUtils.copyProperties(getShippingRate(), record);
            setShippingRate(record);
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void insertShippingRate() throws DataAccessException {
        insertShippingRate(false);
    }

    public void insertShippingRate(boolean commit) throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
            session.save(getShippingRate());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void updateShippingRate() throws DataAccessException {
        updateShippingRate(false);
    }

    public void updateShippingRate(boolean commit) throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
            session.update(getShippingRate());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void deleteShippingRate() throws DataAccessException {
        deleteShippingRate(false);
    }

    public void deleteShippingRate(boolean commit) throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
            setShippingRate((ShippingRate) session.load(ShippingRate.class,
                    new Integer(getShippingRate().getShippingRateID())));
            session.delete(getShippingRate());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }
}
