CREATE TABLE sscShippingRule (
  shippingRuleID INT(11) NOT NULL AUTO_INCREMENT,
  code VARCHAR(100) NOT NULL DEFAULT '',
  name VARCHAR(255) DEFAULT NULL,
  description TEXT,
  isActive TINYINT(4) NOT NULL DEFAULT '1',
  ruleType VARCHAR(32) DEFAULT NULL,
  ruleOrder INT(11) NOT NULL DEFAULT '0',
  extra1 VARCHAR(255) DEFAULT NULL,
  extra2 VARCHAR(255) DEFAULT NULL,
  extra3 VARCHAR(255) DEFAULT NULL,
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.shipping.ShippingRuleBean',
  PRIMARY KEY  (shippingRuleID),
  UNIQUE KEY shippingRuleCodeUKey (code)
);

CREATE INDEX IDX_sscShippingRule_name ON sscShippingRule (name);
CREATE INDEX IDX_sscShippingRule_ruleOrder ON sscShippingRule (ruleOrder);

CREATE TABLE sscSKUShippingRule (
  skuShippingRuleID INT(11) NOT NULL AUTO_INCREMENT,
  skuID INT(11) NOT NULL DEFAULT '0',
  shippingRuleID INT(11) NOT NULL DEFAULT '0',
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.shipping.SKUShippingRuleBean',
  PRIMARY KEY  (skuShippingRuleID),
  KEY skuIDIndex (skuID),
  FOREIGN KEY (`skuID`) REFERENCES `sscSKU` (`skuID`) ON DELETE CASCADE,
  KEY shippingRuleIDIndex (shippingRuleID),
  UNIQUE KEY shippingRuleCodeUKey (skuID, shippingRuleID),
  FOREIGN KEY (`shippingRuleID`) REFERENCES `sscShippingRule` (`shippingRuleID`) ON DELETE CASCADE
);

CREATE TABLE sscShippingRuleRange (
  shippingRuleRangeID INT(11) NOT NULL AUTO_INCREMENT,
  shippingRuleID INT(11) NOT NULL,
  floor DECIMAL(10,4) NOT NULL DEFAULT '0',
  ceiling DECIMAL(10,4) NOT NULL DEFAULT '0',
  isActive TINYINT(4) NOT NULL DEFAULT '1',
  discountType VARCHAR(32) DEFAULT NULL,
  discount DECIMAL(10,4) NOT NULL DEFAULT '0',
  pkgDescription TEXT,
  pkgWeight DECIMAL(10,4) NOT NULL DEFAULT '0',
  pkgLength DECIMAL(10,4) NOT NULL DEFAULT '0',
  pkgHeight DECIMAL(10,4) NOT NULL DEFAULT '0',
  pkgWidth DECIMAL(10,4) NOT NULL DEFAULT '0',
  pkgExtra1 VARCHAR(255) DEFAULT NULL,
  pkgExtra2 VARCHAR(255) DEFAULT NULL,
  pkgExtra3 VARCHAR(255) DEFAULT NULL,
  pkgExtra4 VARCHAR(255) DEFAULT NULL,
  pkgExtra5 VARCHAR(255) DEFAULT NULL,
  limitMethods TEXT,
  limitType VARCHAR(32) DEFAULT NULL,
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.shipping.ShippingRuleRangeBean',
  PRIMARY KEY  (shippingRuleRangeID),
  KEY shippingRuleIDIndex (shippingRuleID),
  FOREIGN KEY (`shippingRuleID`) REFERENCES `sscShippingRule` (`shippingRuleID`) ON DELETE CASCADE
);

CREATE TABLE sscOrderShippingRule (  
  orderShippingRuleID INT(11) NOT NULL AUTO_INCREMENT,
  orderID INT(11) NOT NULL DEFAULT '0',
  orderItemID INT(11) NULL,
  total DECIMAL(10,4) DEFAULT NULL,
  shippingRuleID INT(11) NOT NULL DEFAULT '0',
  code VARCHAR(100) NOT NULL DEFAULT '',
  name VARCHAR(255) DEFAULT NULL,
  ruleType VARCHAR(32) DEFAULT NULL,
  floor DECIMAL(10,4) DEFAULT NULL,
  ceiling DECIMAL(10,4) DEFAULT NULL,
  discountType VARCHAR(32) DEFAULT NULL,
  discount DECIMAL(10,4) NOT NULL DEFAULT '0',
  pkgDescription TEXT,
  pkgWeight DECIMAL(10,4) NOT NULL DEFAULT '0',
  pkgLength DECIMAL(10,4) NOT NULL DEFAULT '0',
  pkgHeight DECIMAL(10,4) NOT NULL DEFAULT '0',
  pkgWidth DECIMAL(10,4) NOT NULL DEFAULT '0',
  limitMethods TEXT,
  limitType VARCHAR(32) DEFAULT NULL,
  extra1 VARCHAR(255) DEFAULT NULL,
  extra2 VARCHAR(255) DEFAULT NULL,
  extra3 VARCHAR(255) DEFAULT NULL,
  pkgExtra1 VARCHAR(255) DEFAULT NULL,
  pkgExtra2 VARCHAR(255) DEFAULT NULL,
  pkgExtra3 VARCHAR(255) DEFAULT NULL,
  pkgExtra4 VARCHAR(255) DEFAULT NULL,
  pkgExtra5 VARCHAR(255) DEFAULT NULL,
  subtotal DECIMAL(10,4) DEFAULT NULL,
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.order.OrderShippingRuleBean',
  PRIMARY KEY  (orderShippingRuleID),
  KEY orderIDIndex (orderID),
  FOREIGN KEY (`orderID`) REFERENCES `sscOrder` (`orderID`) ON DELETE CASCADE
);

CREATE INDEX IDX_sscOrderShippingRule_orderItemID ON sscOrderShippingRule (orderItemID);
CREATE INDEX IDX_sscOrderShippingRule_total ON sscOrderShippingRule (total);
CREATE INDEX IDX_sscOrderShippingRule_code ON sscOrderShippingRule (code);
CREATE INDEX IDX_sscOrderShippingRule_name ON sscOrderShippingRule (name);
